from typing import ClassVar, TypeVar
from scipy.special import logsumexp as logsumexp
from abc import ABCMeta, abstractmethod as abstractmethod
from .base import BaseEstimator, ClassifierMixin
from .preprocessing import (
    binarize,
    LabelBinarizer as LabelBinarizer,
    label_binarize as label_binarize,
)
from .utils.validation import (
    check_is_fitted as check_is_fitted,
    check_non_negative as check_non_negative,
)
from numpy import ndarray
from numbers import Real as Real, Integral as Integral
from .utils.extmath import safe_sparse_dot as safe_sparse_dot
from .utils._param_validation import (
    Interval as Interval,
    Hidden as Hidden,
    StrOptions as StrOptions,
)
from ._typing import MatrixLike, ArrayLike, Float, Int

_BaseDiscreteNB_Self = TypeVar("_BaseDiscreteNB_Self", bound="_BaseDiscreteNB")
GaussianNB_Self = TypeVar("GaussianNB_Self", bound="GaussianNB")
CategoricalNB_Self = TypeVar("CategoricalNB_Self", bound="CategoricalNB")


# Author: Vincent Michel <vincent.michel@inria.fr>
#         Minor fixes by Fabian Pedregosa
#         Amit Aides <amitibo@tx.technion.ac.il>
#         Yehuda Finkelstein <yehudaf@tx.technion.ac.il>
#         Lars Buitinck
#         Jan Hendrik Metzen <jhm@informatik.uni-bremen.de>
#         (parts based on earlier work by Mathieu Blondel)
#
# License: BSD 3 clause
import warnings

import numpy as np

__all__ = [
    "BernoulliNB",
    "GaussianNB",
    "MultinomialNB",
    "ComplementNB",
    "CategoricalNB",
]


class _BaseNB(ClassifierMixin, BaseEstimator, metaclass=ABCMeta):
    def predict_joint_log_proba(self, X: MatrixLike) -> ndarray:
        ...

    def predict(self, X: MatrixLike) -> ndarray:
        ...

    def predict_log_proba(self, X: MatrixLike) -> ndarray:
        ...

    def predict_proba(self, X: MatrixLike) -> ndarray:
        ...


class GaussianNB(_BaseNB):
    theta_: ndarray = ...
    var_: ndarray = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    epsilon_: float = ...
    classes_: ndarray = ...
    class_prior_: ndarray = ...
    class_count_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self, *, priors: None | ArrayLike = None, var_smoothing: Float = 1e-9
    ) -> None:
        ...

    def fit(
        self: GaussianNB_Self,
        X: MatrixLike,
        y: ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> GaussianNB_Self:
        ...

    def partial_fit(
        self: GaussianNB_Self,
        X: MatrixLike,
        y: ArrayLike,
        classes: None | ArrayLike = None,
        sample_weight: None | ArrayLike = None,
    ) -> GaussianNB_Self:
        ...


class _BaseDiscreteNB(_BaseNB):

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        alpha: Float = 1.0,
        fit_prior: bool = True,
        class_prior=None,
        force_alpha: str = "warn",
    ) -> None:
        ...

    def partial_fit(
        self: _BaseDiscreteNB_Self,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        classes: None | ArrayLike = None,
        sample_weight: None | ArrayLike = None,
    ) -> MultinomialNB | _BaseDiscreteNB_Self:
        ...

    def fit(
        self: _BaseDiscreteNB_Self,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> _BaseDiscreteNB_Self | BernoulliNB | ComplementNB:
        ...


class MultinomialNB(_BaseDiscreteNB):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    feature_log_prob_: ndarray = ...
    feature_count_: ndarray = ...
    classes_: ndarray = ...
    class_log_prior_: ndarray = ...
    class_count_: ndarray = ...

    def __init__(
        self,
        *,
        alpha: float | ArrayLike = 1.0,
        force_alpha: str | bool = "warn",
        fit_prior: bool = True,
        class_prior: None | ArrayLike = None,
    ) -> None:
        ...


class ComplementNB(_BaseDiscreteNB):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    feature_log_prob_: ndarray = ...
    feature_count_: ndarray = ...
    feature_all_: ndarray = ...
    classes_: ndarray = ...
    class_log_prior_: ndarray = ...
    class_count_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        alpha: float | ArrayLike = 1.0,
        force_alpha: str | bool = "warn",
        fit_prior: bool = True,
        class_prior: None | ArrayLike = None,
        norm: bool = False,
    ) -> None:
        ...


class BernoulliNB(_BaseDiscreteNB):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    feature_log_prob_: ndarray = ...
    feature_count_: ndarray = ...
    classes_: ndarray = ...
    class_log_prior_: ndarray = ...
    class_count_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        alpha: float | ArrayLike = 1.0,
        force_alpha: str | bool = "warn",
        binarize: None | Float = 0.0,
        fit_prior: bool = True,
        class_prior: None | ArrayLike = None,
    ) -> None:
        ...


class CategoricalNB(_BaseDiscreteNB):
    n_categories_: ndarray = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    feature_log_prob_: list[ArrayLike] = ...
    classes_: ndarray = ...
    class_log_prior_: ndarray = ...
    class_count_: ndarray = ...
    category_count_: list[ArrayLike] = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        alpha: Float = 1.0,
        force_alpha: str | bool = "warn",
        fit_prior: bool = True,
        class_prior: None | ArrayLike = None,
        min_categories: None | ArrayLike | Int = None,
    ) -> None:
        ...

    def fit(
        self: CategoricalNB_Self,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> CategoricalNB_Self:
        ...

    def partial_fit(
        self: CategoricalNB_Self,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        classes: None | ArrayLike = None,
        sample_weight: None | ArrayLike = None,
    ) -> CategoricalNB_Self:
        ...



from typing import Any, List, Optional, Sequence, Union

from django.apps.config import AppConfig
from django.contrib.admin.options import BaseModelAdmin
from django.core.checks.messages import CheckMessage, Error

_CheckError = Union[str, Error]

def check_admin_app(
    app_configs: Optional[Sequence[AppConfig]] = ..., **kwargs: Any
) -> List[_CheckError]: ...
def check_dependencies(**kwargs: Any) -> List[_CheckError]: ...

class BaseModelAdminChecks:
    def check(
        self, admin_obj: BaseModelAdmin[Any], **kwargs: Any
    ) -> List[CheckMessage]: ...

class ModelAdminChecks(BaseModelAdminChecks): ...
class InlineModelAdminChecks(BaseModelAdminChecks): ...

def must_be(type: Any, option: Any, obj: Any, id: Any) -> Any: ...
def must_inherit_from(parent: Any, option: Any, obj: Any, id: Any) -> Any: ...
def refer_to_missing_field(
    field: Any, option: Any, model: Any, obj: Any, id: Any
) -> Any: ...



from numpy.typing import ArrayLike

from copy import deepcopy
from functools import lru_cache
from random import choice

from ...classes.graph import Graph
from ...utils import not_implemented_for

__all__ = ["lukes_partitioning"]

D_EDGE_W: str = ...
D_EDGE_VALUE: float = ...
D_NODE_W: str = ...
D_NODE_VALUE: int = ...
PKEY: str = ...
CLUSTER_EVAL_CACHE_SIZE: int = ...

def lukes_partitioning(
    G: Graph, max_size: int, node_weight=None, edge_weight=None
) -> ArrayLike: ...



from ...classes.graph import Graph
from .mixing import (
    attribute_mixing_matrix,
    degree_mixing_matrix,
)
from .pairs import node_degree_xy

__all__ = [
    "degree_pearson_correlation_coefficient",
    "degree_assortativity_coefficient",
    "attribute_assortativity_coefficient",
    "numeric_assortativity_coefficient",
]

def degree_assortativity_coefficient(
    G: Graph, x="out", y="in", weight=None, nodes=None
) -> float: ...
def degree_pearson_correlation_coefficient(
    G: Graph, x="out", y="in", weight=None, nodes=None
) -> float: ...
def attribute_assortativity_coefficient(
    G: Graph, attribute: str, nodes=None
) -> float: ...
def numeric_assortativity_coefficient(
    G: Graph, attribute: str, nodes=None
) -> float: ...
def attribute_ac(M): ...



from ...classes.graph import Graph
from ...utils import not_implemented_for

from .edmondskarp import edmonds_karp
from .utils import build_residual_network

default_flow_func = ...

__all__ = ["gomory_hu_tree"]

def gomory_hu_tree(G: Graph, capacity: str = "capacity", flow_func=None): ...



from collections import abc
from collections.abc import Sequence
import datetime
from io import BytesIO
from types import TracebackType
from typing import (
    Literal,
    overload,
)

from pandas.core.frame import DataFrame
from typing_extensions import Self

from pandas._typing import (
    CompressionOptions,
    FilePath,
    HashableT,
    HashableT1,
    HashableT2,
    HashableT3,
    ReadBuffer,
    StataDateFormat,
    StorageOptions,
    WriteBuffer,
)

@overload
def read_stata(
    filepath_or_buffer: FilePath | ReadBuffer[bytes],
    *,
    convert_dates: bool = ...,
    convert_categoricals: bool = ...,
    index_col: str | None = ...,
    convert_missing: bool = ...,
    preserve_dtypes: bool = ...,
    columns: list[HashableT] | None = ...,
    order_categoricals: bool = ...,
    chunksize: int | None = ...,
    iterator: Literal[True],
    compression: CompressionOptions = ...,
    storage_options: StorageOptions = ...,
) -> StataReader:
    """
Read Stata file into DataFrame.

Parameters
----------
filepath_or_buffer : str, path object or file-like object
    Any valid string path is acceptable. The string could be a URL. Valid
    URL schemes include http, ftp, s3, and file. For file URLs, a host is
    expected. A local file could be: ``file://localhost/path/to/table.dta``.

    If you want to pass in a path object, pandas accepts any ``os.PathLike``.

    By file-like object, we refer to objects with a ``read()`` method,
    such as a file handle (e.g. via builtin ``open`` function)
    or ``StringIO``.
convert_dates : bool, default True
    Convert date variables to DataFrame time values.
convert_categoricals : bool, default True
    Read value labels and convert columns to Categorical/Factor variables.
index_col : str, optional
    Column to set as index.
convert_missing : bool, default False
    Flag indicating whether to convert missing values to their Stata
    representations.  If False, missing values are replaced with nan.
    If True, columns containing missing values are returned with
    object data types and missing values are represented by
    StataMissingValue objects.
preserve_dtypes : bool, default True
    Preserve Stata datatypes. If False, numeric data are upcast to pandas
    default types for foreign data (float64 or int64).
columns : list or None
    Columns to retain.  Columns will be returned in the given order.  None
    returns all columns.
order_categoricals : bool, default True
    Flag indicating whether converted categorical data are ordered.
chunksize : int, default None
    Return StataReader object for iterations, returns chunks with
    given number of lines.
iterator : bool, default False
    Return StataReader object.
compression : str or dict, default 'infer'
    For on-the-fly decompression of on-disk data. If 'infer' and 'filepath_or_buffer' is
    path-like, then detect compression from the following extensions: '.gz',
    '.bz2', '.zip', '.xz', '.zst', '.tar', '.tar.gz', '.tar.xz' or '.tar.bz2'
    (otherwise no compression).
    If using 'zip' or 'tar', the ZIP file must contain only one data file to be read in.
    Set to ``None`` for no decompression.
    Can also be a dict with key ``'method'`` set
    to one of {``'zip'``, ``'gzip'``, ``'bz2'``, ``'zstd'``, ``'tar'``} and other
    key-value pairs are forwarded to
    ``zipfile.ZipFile``, ``gzip.GzipFile``,
    ``bz2.BZ2File``, ``zstandard.ZstdDecompressor`` or
    ``tarfile.TarFile``, respectively.
    As an example, the following could be passed for Zstandard decompression using a
    custom compression dictionary:
    ``compression={'method': 'zstd', 'dict_data': my_compression_dict}``.

    .. versionadded:: 1.5.0
        Added support for `.tar` files.
storage_options : dict, optional
    Extra options that make sense for a particular storage connection, e.g.
    host, port, username, password, etc. For HTTP(S) URLs the key-value pairs
    are forwarded to ``urllib.request.Request`` as header options. For other
    URLs (e.g. starting with "s3://", and "gcs://") the key-value pairs are
    forwarded to ``fsspec.open``. Please see ``fsspec`` and ``urllib`` for more
    details, and for more examples on storage options refer `here
    <https://pandas.pydata.org/docs/user_guide/io.html?
    highlight=storage_options#reading-writing-remote-files>`_.

Returns
-------
DataFrame or StataReader

See Also
--------
io.stata.StataReader : Low-level reader for Stata data files.
DataFrame.to_stata: Export Stata data files.

Notes
-----
Categorical variables read through an iterator may not have the same
categories and dtype. This occurs when  a variable stored in a DTA
file is associated to an incomplete set of value labels that only
label a strict subset of the values.

Examples
--------

Creating a dummy stata for this example

>>> df = pd.DataFrame({'animal': ['falcon', 'parrot', 'falcon', 'parrot'],
...                     'speed': [350, 18, 361, 15]})  # doctest: +SKIP
>>> df.to_stata('animals.dta')  # doctest: +SKIP

Read a Stata dta file:

>>> df = pd.read_stata('animals.dta')  # doctest: +SKIP

Read a Stata dta file in 10,000 line chunks:

>>> values = np.random.randint(0, 10, size=(20_000, 1), dtype="uint8")  # doctest: +SKIP
>>> df = pd.DataFrame(values, columns=["i"])  # doctest: +SKIP
>>> df.to_stata('filename.dta')  # doctest: +SKIP

>>> with pd.read_stata('filename.dta', chunksize=10000) as itr: # doctest: +SKIP
>>>     for chunk in itr:
...         # Operate on a single chunk, e.g., chunk.mean()
...         pass  # doctest: +SKIP
    """
    pass
@overload
def read_stata(
    filepath_or_buffer: FilePath | ReadBuffer[bytes],
    *,
    convert_dates: bool = ...,
    convert_categoricals: bool = ...,
    index_col: str | None = ...,
    convert_missing: bool = ...,
    preserve_dtypes: bool = ...,
    columns: list[HashableT] | None = ...,
    order_categoricals: bool = ...,
    chunksize: int,
    iterator: bool = ...,
    compression: CompressionOptions = ...,
    storage_options: StorageOptions = ...,
) -> StataReader: ...
@overload
def read_stata(
    filepath_or_buffer: FilePath | ReadBuffer[bytes],
    *,
    convert_dates: bool = ...,
    convert_categoricals: bool = ...,
    index_col: str | None = ...,
    convert_missing: bool = ...,
    preserve_dtypes: bool = ...,
    columns: list[HashableT] | None = ...,
    order_categoricals: bool = ...,
    chunksize: None = ...,
    iterator: Literal[False] = ...,
    compression: CompressionOptions = ...,
    storage_options: StorageOptions = ...,
) -> DataFrame: ...

class StataParser:
    def __init__(self) -> None: ...

class StataReader(StataParser, abc.Iterator):
    col_sizes: list[int] = ...
    path_or_buf: BytesIO = ...
    def __init__(
        self,
        path_or_buf: FilePath | ReadBuffer[bytes],
        convert_dates: bool = ...,
        convert_categoricals: bool = ...,
        index_col: str | None = ...,
        convert_missing: bool = ...,
        preserve_dtypes: bool = ...,
        columns: Sequence[str] | None = ...,
        order_categoricals: bool = ...,
        chunksize: int | None = ...,
        compression: CompressionOptions = ...,
        storage_options: StorageOptions = ...,
    ) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_value: BaseException | None,
        traceback: TracebackType | None,
    ) -> None: ...
    def close(self) -> None: ...
    def __next__(self) -> DataFrame: ...
    def get_chunk(self, size: int | None = ...) -> DataFrame: ...
    def read(
        self,
        nrows: int | None = ...,
        convert_dates: bool | None = ...,
        convert_categoricals: bool | None = ...,
        index_col: str | None = ...,
        convert_missing: bool | None = ...,
        preserve_dtypes: bool | None = ...,
        columns: list[str] | None = ...,
        order_categoricals: bool | None = ...,
    ): ...
    @property
    def data_label(self) -> str: ...
    def variable_labels(self) -> dict[str, str]: ...
    def value_labels(self) -> dict[str, dict[float, str]]: ...

class StataWriter(StataParser):
    def __init__(
        self,
        fname: FilePath | WriteBuffer[bytes],
        data: DataFrame,
        convert_dates: dict[HashableT1, StataDateFormat] | None = ...,
        write_index: bool = ...,
        byteorder: str | None = ...,
        time_stamp: datetime.datetime | None = ...,
        data_label: str | None = ...,
        variable_labels: dict[HashableT2, str] | None = ...,
        compression: CompressionOptions = ...,
        storage_options: StorageOptions = ...,
        *,
        value_labels: dict[HashableT3, dict[float, str]] | None = ...,
    ) -> None: ...
    def write_file(self) -> None: ...



"""
This type stub file was generated by pyright.
"""

import sys

"""
Utilities for working with external processes.
"""
if sys.platform == 'win32':
    ...
else:
    ...
class FindCmdError(Exception):
    ...


def find_cmd(cmd):
    """Find absolute path to executable cmd in a cross platform manner.

    This function tries to determine the full path to a command line program
    using `which` on Unix/Linux/OS X and `win32api` on Windows.  Most of the
    time it will use the version that is first on the users `PATH`.

    Warning, don't use this to find IPython command line programs as there
    is a risk you will find the wrong one.  Instead find those using the
    following code and looking for the application itself::

        import sys
        argv = [sys.executable, '-m', 'IPython']

    Parameters
    ----------
    cmd : str
        The command line program to look for.
    """
    ...

def abbrev_cwd():
    """ Return abbreviated version of cwd, e.g. d:mydir """
    ...




"""
This type stub file was generated by pyright.
"""

"""Simple function to call to get the current InteractiveShell instance
"""
def get_ipython(): # -> None:
    """Get the global InteractiveShell instance.

    Returns None if no InteractiveShell instance is registered.
    """
    ...




from typing import Callable

class Substitution:
    def __init__(self, *args, **kwargs) -> None: ...
    def __call__(self, func: Callable) -> Callable: ...
    def update(self, *args, **kwargs) -> None: ...

class _ArtistKwdocLoader(dict):
    def __missing__(self, key: str) -> str: ...

class _ArtistPropertiesSubstitution(Substitution):
    def __init__(self) -> None: ...
    def __call__(self, obj: Callable) -> Callable: ...

def copy(source: Callable) -> Callable: ...

dedent_interpd: _ArtistPropertiesSubstitution = ...
interpd: _ArtistPropertiesSubstitution = ...



from vispy.visuals.collections.collection import Collection
from numpy.typing import ArrayLike, NDArray

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2014, Nicolas P. Rougier
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

import math
import numpy as np
from ...gloo import Texture2D, VertexBuffer, IndexBuffer
from .util import dtype_reduce
from .array_list import ArrayList

def next_power_of_2(n): ...

class Item(object):
    def __init__(
        self,
        parent: Collection,
        key: int,
        vertices: ArrayLike,
        indices: ArrayLike,
        uniforms: ArrayLike,
    ): ...
    @property
    def vertices(self): ...
    @vertices.setter
    def vertices(self, data): ...
    @property
    def indices(self): ...
    @indices.setter
    def indices(self, data): ...
    @property
    def uniforms(self): ...
    @uniforms.setter
    def uniforms(self, data): ...
    def __getitem__(self, key): ...
    def __setitem__(self, key, value): ...
    def __str__(self): ...

class BaseCollection(object):
    def __init__(self, vtype, utype=None, itype=None): ...
    def __len__(self): ...
    @property
    def vtype(self): ...
    @property
    def itype(self): ...
    @property
    def utype(self): ...
    def append(
        self,
        vertices: NDArray,
        uniforms: NDArray | None = None,
        indices: NDArray | None = None,
        itemsize: int | tuple | ArrayLike | None = None,
    ): ...
    def __delitem__(self, index): ...
    def __getitem__(self, key): ...
    def __setitem__(self, key, data): ...
    def _compute_texture_shape(self, size=1): ...
    def _update(self): ...



"""
This type stub file was generated by pyright.
"""

"""cli-specific implementation of process utilities.

cli - Common Language Infrastructure for IronPython. Code
      can run on any operating system. Check os.name for os-
      specific settings.

This file is only meant to be imported by process.py, not by end-users.

This file is largely untested. To become a full drop-in process
interface for IronPython will probably require you to help fill
in the details. 
"""
def system(cmd): # -> None:
    """
    system(cmd) should work in a cli environment on Mac OSX, Linux,
    and Windows
    """
    ...

def getoutput(cmd):
    """
    getoutput(cmd) should work in a cli environment on Mac OSX, Linux,
    and Windows
    """
    ...

def check_pid(pid):
    """
    Check if a process with the given PID (pid) exists
    """
    ...




# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

# NOTE: modules in this package that start with one underscore are
# autogenerated, and should not be edited.

from __future__ import division as division

import os as os

from ...util import config as config, logger as logger

from ._constants import *  # noqa
from ._proxy import BaseGLProxy as BaseGLProxy

# Variable that will hold the module corresponding to the current backend
# This variable is used in our proxy classes to call the right functions.
current_backend: None = ...

class MainProxy(BaseGLProxy):
    def __call__(self, funcname, returns, *args): ...

# Instantiate proxy objects
proxy = ...

def use_gl(target: str | None = None): ...
def _clear_namespace(): ...
def _copy_gl_functions(source, dest, constants=False, debug=False): ...
def _arg_repr(arg): ...
def make_debug_wrapper(fn): ...
def check_error(when: str = "periodic check"): ...
def _fix_osmesa_gl_lib_if_testing(): ...

# Load default gl backend
from . import gl2 as default_backend  # noqa



import itertools
from collections import defaultdict, deque

from ...utils import arbitrary_element, py_random_state
from ...classes.graph import Graph

__all__ = [
    "greedy_color",
    "strategy_connected_sequential",
    "strategy_connected_sequential_bfs",
    "strategy_connected_sequential_dfs",
    "strategy_independent_set",
    "strategy_largest_first",
    "strategy_random_sequential",
    "strategy_saturation_largest_first",
    "strategy_smallest_last",
]

def strategy_largest_first(G: Graph, colors): ...
@py_random_state(2)
def strategy_random_sequential(G: Graph, colors, seed=None): ...
def strategy_smallest_last(G: Graph, colors): ...
def strategy_independent_set(G: Graph, colors): ...
def strategy_connected_sequential_bfs(G: Graph, colors): ...
def strategy_connected_sequential_dfs(G: Graph, colors): ...
def strategy_connected_sequential(G: Graph, colors, traversal="bfs"): ...
def strategy_saturation_largest_first(G: Graph, colors): ...

#: Dictionary mapping name of a strategy as a string to the strategy function.
STRATEGIES: dict = ...

def greedy_color(G: Graph, strategy="largest_first", interchange: bool = False): ...

# Tools for coloring with interchanges
class _Node:
    __slots__: list = ...

    def __init__(self, node_id, n): ...
    def __repr__(self): ...
    def assign_color(self, adj_entry, color): ...
    def clear_color(self, adj_entry, color): ...
    def iter_neighbors(self): ...
    def iter_neighbors_color(self, color): ...

class _AdjEntry:
    __slots__: list = ...

    def __init__(self, node_id): ...
    def __repr__(self): ...



from typing import Callable, ClassVar, Literal, TypeVar
from ..utils.validation import check_is_fitted as check_is_fitted
from scipy import special as special, stats
from ._base import SelectorMixin
from numpy import ndarray
from ..utils.extmath import safe_sparse_dot as safe_sparse_dot, row_norms as row_norms
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from numbers import Integral as Integral, Real as Real
from joblib.memory import MemorizedFunc
from ..base import BaseEstimator
from scipy.sparse import issparse as issparse
from .._typing import MatrixLike, ArrayLike, Int, Float
from ..utils import (
    as_float_array as as_float_array,
    check_array as check_array,
    check_X_y as check_X_y,
    safe_sqr as safe_sqr,
    safe_mask as safe_mask,
)
from ..preprocessing import LabelBinarizer as LabelBinarizer

_BaseFilter_Self = TypeVar("_BaseFilter_Self", bound="_BaseFilter")


# Authors: V. Michel, B. Thirion, G. Varoquaux, A. Gramfort, E. Duchesnay.
#          L. Buitinck, A. Joly
# License: BSD 3 clause


import numpy as np
import warnings


######################################################################
# Scoring functions


# The following function is a rewriting of scipy.stats.f_oneway
# Contrary to the scipy.stats.f_oneway implementation it does not
# copy the data while keeping the inputs unchanged.
def f_oneway(*args) -> tuple[ndarray, ndarray] | tuple[float, float]:
    ...


def f_classif(X: MatrixLike | ArrayLike, y: ArrayLike) -> tuple[ndarray, ndarray]:
    ...


def chi2(X: MatrixLike | ArrayLike, y: ArrayLike) -> tuple[ndarray, ndarray]:
    ...


def r_regression(
    X: MatrixLike | ArrayLike,
    y: ArrayLike,
    *,
    center: bool = True,
    force_finite: bool = True
) -> ndarray:
    ...


def f_regression(
    X: MatrixLike | ArrayLike,
    y: ArrayLike,
    *,
    center: bool = True,
    force_finite: bool = True
) -> tuple[ndarray, ndarray]:
    ...


######################################################################
# Base classes


class _BaseFilter(SelectorMixin, BaseEstimator):

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(self, score_func: Callable | MemorizedFunc) -> None:
        ...

    def fit(self: _BaseFilter_Self, X: MatrixLike, y: ArrayLike) -> _BaseFilter_Self:
        ...


######################################################################
# Specific filters
######################################################################
class SelectPercentile(_BaseFilter):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    pvalues_: ArrayLike = ...
    scores_: ArrayLike = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self, score_func: Callable | MemorizedFunc = ..., *, percentile: Int = 10
    ) -> None:
        ...


class SelectKBest(_BaseFilter):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    pvalues_: ArrayLike = ...
    scores_: ArrayLike = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(self, score_func: Callable = ..., *, k: str | int = 10) -> None:
        ...


class SelectFpr(_BaseFilter):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    pvalues_: ArrayLike = ...
    scores_: ArrayLike = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(self, score_func: Callable = ..., *, alpha: Float = 5e-2) -> None:
        ...


class SelectFdr(_BaseFilter):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    pvalues_: ArrayLike = ...
    scores_: ArrayLike = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(self, score_func: Callable = ..., *, alpha: Float = 5e-2) -> None:
        ...


class SelectFwe(_BaseFilter):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    pvalues_: ArrayLike = ...
    scores_: ArrayLike = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(self, score_func: Callable = ..., *, alpha: Float = 5e-2) -> None:
        ...


######################################################################
# Generic filter
######################################################################

# TODO this class should fit on either p-values or scores,
# depending on the mode.
class GenericUnivariateSelect(_BaseFilter):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    pvalues_: ArrayLike = ...
    scores_: ArrayLike = ...

    _selection_modes: ClassVar[dict] = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        score_func: Callable = ...,
        *,
        mode: Literal[
            "percentile", "k_best", "fpr", "fdr", "fwe", "percentile"
        ] = "percentile",
        param: float | str | int = 1e-5
    ) -> None:
        ...



"""
This type stub file was generated by pyright.
"""

import functools
import types
import IPython.utils.colorable as colorable
from types import TracebackType
from typing import Any, List, Optional, Tuple

"""
Verbose and colourful traceback formatting.

**ColorTB**

I've always found it a bit hard to visually parse tracebacks in Python.  The
ColorTB class is a solution to that problem.  It colors the different parts of a
traceback in a manner similar to what you would expect from a syntax-highlighting
text editor.

Installation instructions for ColorTB::

    import sys,ultratb
    sys.excepthook = ultratb.ColorTB()

**VerboseTB**

I've also included a port of Ka-Ping Yee's "cgitb.py" that produces all kinds
of useful info when a traceback occurs.  Ping originally had it spit out HTML
and intended it for CGI programmers, but why should they have all the fun?  I
altered it to spit out colored text to the terminal.  It's a bit overwhelming,
but kind of neat, and maybe useful for long-running programs that you believe
are bug-free.  If a crash *does* occur in that type of program you want details.
Give it a shot--you'll love it or you'll hate it.

.. note::

  The Verbose mode prints the variables currently visible where the exception
  happened (shortening their strings if too long). This can potentially be
  very slow, if you happen to have a huge data structure whose string
  representation is complex to compute. Your computer may appear to freeze for
  a while with cpu usage at 100%. If this occurs, you can cancel the traceback
  with Ctrl-C (maybe hitting it more than once).

  If you encounter this kind of situation often, you may want to use the
  Verbose_novars mode instead of the regular Verbose, which avoids formatting
  variables (but otherwise includes the information and context given by
  Verbose).

.. note::

  The verbose mode print all variables in the stack, which means it can
  potentially leak sensitive information like access keys, or unencrypted
  password.

Installation instructions for VerboseTB::

    import sys,ultratb
    sys.excepthook = ultratb.VerboseTB()

Note:  Much of the code in this module was lifted verbatim from the standard
library module 'traceback.py' and Ka-Ping Yee's 'cgitb.py'.

Color schemes
-------------

The colors are defined in the class TBTools through the use of the
ColorSchemeTable class. Currently the following exist:

  - NoColor: allows all of this module to be used in any terminal (the color
    escapes are just dummy blank strings).

  - Linux: is meant to look good in a terminal like the Linux console (black
    or very dark background).

  - LightBG: similar to Linux but swaps dark/light colors to be more readable
    in light background terminals.

  - Neutral: a neutral color scheme that should be readable on both light and
    dark background

You can implement other color schemes easily, the syntax is fairly
self-explanatory. Please send back new schemes you develop to the author for
possible inclusion in future releases.

Inheritance diagram:

.. inheritance-diagram:: IPython.core.ultratb
   :parts: 3
"""
INDENT_SIZE = ...
DEFAULT_SCHEME = ...
FAST_THRESHOLD = ...
@functools.lru_cache()
def count_lines_in_py_file(filename: str) -> int:
    """
    Given a filename, returns the number of lines in the file
    if it ends with the extension ".py". Otherwise, returns 0.
    """
    ...

def get_line_number_of_frame(frame: types.FrameType) -> int:
    """
    Given a frame object, returns the total number of lines in the file
    containing the frame's code object, or the number of lines in the
    frame's source code if the file is not available.

    Parameters
    ----------
    frame : FrameType
        The frame object whose line number is to be determined.

    Returns
    -------
    int
        The total number of lines in the file containing the frame's
        code object, or the number of lines in the frame's source code
        if the file is not available.
    """
    ...

class TBTools(colorable.Colorable):
    """Basic tools used by all traceback printer classes."""
    tb_offset = ...
    def __init__(self, color_scheme=..., call_pdb=..., ostream=..., parent=..., config=..., *, debugger_cls=...) -> None:
        ...
    
    ostream = ...
    def get_parts_of_chained_exception(self, evalue) -> Optional[Tuple[type, BaseException, TracebackType]]:
        ...
    
    def prepare_chained_exception_message(self, cause) -> List[Any]:
        ...
    
    @property
    def has_colors(self) -> bool:
        ...
    
    def set_colors(self, *args, **kw): # -> None:
        """Shorthand access to the color table scheme selector method."""
        ...
    
    def color_toggle(self): # -> None:
        """Toggle between the currently active color scheme and NoColor."""
        ...
    
    def stb2text(self, stb):
        """Convert a structured traceback (a list) to a string."""
        ...
    
    def text(self, etype, value, tb, tb_offset: Optional[int] = ..., context=...):
        """Return formatted traceback.

        Subclasses may override this if they add extra arguments.
        """
        ...
    
    def structured_traceback(self, etype: type, evalue: Optional[BaseException], etb: Optional[TracebackType] = ..., tb_offset: Optional[int] = ..., number_of_lines_of_context: int = ...):
        """Return a list of traceback frames.

        Must be implemented by each class.
        """
        ...
    


class ListTB(TBTools):
    """Print traceback information from a traceback list, with optional color.

    Calling requires 3 arguments: (etype, evalue, elist)
    as would be obtained by::

      etype, evalue, tb = sys.exc_info()
      if tb:
        elist = traceback.extract_tb(tb)
      else:
        elist = None

    It can thus be used by programs which need to process the traceback before
    printing (such as console replacements based on the code module from the
    standard library).

    Because they are meant to be called without a full traceback (only a
    list), instances of this class can't call the interactive pdb debugger."""
    def __call__(self, etype, value, elist): # -> None:
        ...
    
    def structured_traceback(self, etype: type, evalue: Optional[BaseException], etb: Optional[TracebackType] = ..., tb_offset: Optional[int] = ..., context=...):
        """Return a color formatted string with the traceback info.

        Parameters
        ----------
        etype : exception type
            Type of the exception raised.
        evalue : object
            Data stored in the exception
        etb : list | TracebackType | None
            If list: List of frames, see class docstring for details.
            If Traceback: Traceback of the exception.
        tb_offset : int, optional
            Number of frames in the traceback to skip.  If not given, the
            instance evalue is used (set in constructor).
        context : int, optional
            Number of lines of context information to print.

        Returns
        -------
        String with formatted exception.
        """
        ...
    
    def get_exception_only(self, etype, value):
        """Only print the exception type and message, without a traceback.

        Parameters
        ----------
        etype : exception type
        value : exception value
        """
        ...
    
    def show_exception_only(self, etype, evalue): # -> None:
        """Only print the exception type and message, without a traceback.

        Parameters
        ----------
        etype : exception type
        evalue : exception value
        """
        ...
    


class FrameInfo:
    """
    Mirror of stack data's FrameInfo, but so that we can bypass highlighting on
    really long frames.
    """
    description: Optional[str]
    filename: Optional[str]
    lineno: Tuple[int]
    context: Optional[int]
    def __init__(self, description: Optional[str], filename: str, lineno: Tuple[int], frame, code, *, sd=..., context=...) -> None:
        ...
    
    @property
    def variables_in_executing_piece(self):
        ...
    
    @property
    def lines(self):
        ...
    
    @property
    def executing(self): # -> None:
        ...
    


class VerboseTB(TBTools):
    """A port of Ka-Ping Yee's cgitb.py module that outputs color text instead
    of HTML.  Requires inspect and pydoc.  Crazy, man.

    Modified version which optionally strips the topmost entries from the
    traceback, to be used with alternate interpreters (because their own code
    would appear in the traceback)."""
    _tb_highlight = ...
    def __init__(self, color_scheme: str = ..., call_pdb: bool = ..., ostream=..., tb_offset: int = ..., long_header: bool = ..., include_vars: bool = ..., check_cache=..., debugger_cls=..., parent=..., config=...) -> None:
        """Specify traceback offset, headers and color scheme.

        Define how many frames to drop from the tracebacks. Calling it with
        tb_offset=1 allows use of this handler in interpreters which will have
        their own code at the top of the traceback (VerboseTB will first
        remove that frame before printing the traceback info)."""
        ...
    
    def format_record(self, frame_info: FrameInfo):
        """Format a single stack frame"""
        ...
    
    def prepare_header(self, etype: str, long_version: bool = ...):
        ...
    
    def format_exception(self, etype, evalue):
        ...
    
    def format_exception_as_a_whole(self, etype: type, evalue: Optional[BaseException], etb: Optional[TracebackType], number_of_lines_of_context, tb_offset: Optional[int]):
        """Formats the header, traceback and exception message for a single exception.

        This may be called multiple times by Python 3 exception chaining
        (PEP 3134).
        """
        ...
    
    def get_records(self, etb: TracebackType, number_of_lines_of_context: int, tb_offset: int):
        ...
    
    def structured_traceback(self, etype: type, evalue: Optional[BaseException], etb: Optional[TracebackType] = ..., tb_offset: Optional[int] = ..., number_of_lines_of_context: int = ...):
        """Return a nice text document describing the traceback."""
        ...
    
    def debugger(self, force: bool = ...): # -> None:
        """Call up the pdb debugger if desired, always clean up the tb
        reference.

        Keywords:

          - force(False): by default, this routine checks the instance call_pdb
            flag and does not actually invoke the debugger if the flag is false.
            The 'force' option forces the debugger to activate even if the flag
            is false.

        If the call_pdb flag is set, the pdb interactive debugger is
        invoked. In all cases, the self.tb reference to the current traceback
        is deleted to prevent lingering references which hamper memory
        management.

        Note that each call to pdb() does an 'import readline', so if your app
        requires a special setup for the readline completers, you'll have to
        fix that by hand after invoking the exception handler."""
        ...
    
    def handler(self, info=...): # -> None:
        ...
    
    def __call__(self, etype=..., evalue=..., etb=...): # -> None:
        """This hook can replace sys.excepthook (for Python 2.1 or higher)."""
        ...
    


class FormattedTB(VerboseTB, ListTB):
    """Subclass ListTB but allow calling with a traceback.

    It can thus be used as a sys.excepthook for Python > 2.1.

    Also adds 'Context' and 'Verbose' modes, not available in ListTB.

    Allows a tb_offset to be specified. This is useful for situations where
    one needs to remove a number of topmost frames from the traceback (such as
    occurs with python programs that themselves execute other python code,
    like Python shells).  """
    mode: str
    def __init__(self, mode=..., color_scheme=..., call_pdb=..., ostream=..., tb_offset=..., long_header=..., include_vars=..., check_cache=..., debugger_cls=..., parent=..., config=...) -> None:
        ...
    
    def structured_traceback(self, etype, value, tb, tb_offset=..., number_of_lines_of_context=...):
        ...
    
    def stb2text(self, stb):
        """Convert a structured traceback (a list) to a string."""
        ...
    
    def set_mode(self, mode: Optional[str] = ...): # -> None:
        """Switch to the desired mode.

        If mode is not specified, cycles through the available modes."""
        ...
    
    def plain(self): # -> None:
        ...
    
    def context(self): # -> None:
        ...
    
    def verbose(self): # -> None:
        ...
    
    def minimal(self): # -> None:
        ...
    


class AutoFormattedTB(FormattedTB):
    """A traceback printer which can be called on the fly.

    It will find out about exceptions by itself.

    A brief example::

        AutoTB = AutoFormattedTB(mode = 'Verbose',color_scheme='Linux')
        try:
          ...
        except:
          AutoTB()  # or AutoTB(out=logfile) where logfile is an open file object
    """
    def __call__(self, etype=..., evalue=..., etb=..., out=..., tb_offset=...): # -> None:
        """Print out a formatted exception traceback.

        Optional arguments:
          - out: an open file-like object to direct output to.

          - tb_offset: the number of frames to skip over in the stack, on a
          per-call basis (this overrides temporarily the instance's tb_offset
          given at initialization time."""
        ...
    
    def structured_traceback(self, etype: type, evalue: Optional[BaseException], etb: Optional[TracebackType] = ..., tb_offset: Optional[int] = ..., number_of_lines_of_context: int = ...):
        ...
    


class ColorTB(FormattedTB):
    """Shorthand to initialize a FormattedTB in Linux colors mode."""
    def __init__(self, color_scheme=..., call_pdb=..., **kwargs) -> None:
        ...
    


class SyntaxTB(ListTB):
    """Extension which holds some state: the last exception value"""
    def __init__(self, color_scheme=..., parent=..., config=...) -> None:
        ...
    
    def __call__(self, etype, value, elist): # -> None:
        ...
    
    def structured_traceback(self, etype, value, elist, tb_offset=..., context=...):
        ...
    
    def clear_err_state(self): # -> None:
        """Return the current error state and clear it"""
        ...
    
    def stb2text(self, stb):
        """Convert a structured traceback (a list) to a string."""
        ...
    


def text_repr(value): # -> None:
    """Hopefully pretty robust repr equivalent."""
    ...

def eqrepr(value, repr=...):
    ...

def nullrepr(value, repr=...):
    ...




from typing import Callable

def safe_import(mod_name: str, min_version: str | None = ...): ...
def skip_if_installed(package: str) -> Callable: ...
def skip_if_no(package: str, min_version: str | None = ...) -> Callable: ...

skip_if_no_mpl = ...
skip_if_mpl = ...
skip_if_32bit = ...
skip_if_windows = ...
skip_if_has_locale = ...
skip_if_not_us_locale = ...
skip_if_no_scipy = ...
skip_if_no_ne = ...

def skip_if_np_lt(
    ver_str: str, reason: str | None = ..., *args, **kwds
) -> Callable: ...
def parametrize_fixture_doc(*args): ...
def check_file_leaks(func) -> Callable: ...
def async_mark(): ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
import os
from distutils.version import LooseVersion

from vispy.util import use_log_level

def has_matplotlib(version="1.2"): ...
def has_skimage(version="0.11"): ...
def has_backend(backend, has=..., capable=..., out=...): ...



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2014, Nicolas P. Rougier
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

from .raw_point_collection import RawPointCollection
from .agg_point_collection import AggPointCollection

def PointCollection(mode="raw", *args, **kwargs): ...



"""Warning filters and `.env` updater."""

from pathlib import Path
from warnings import filterwarnings, resetwarnings


def filter_warnings_and_update_dotenv():
    """Filter warnings and update PYTHONWARNINGS in `.env` file."""
    filters: list[str] = []
    for filt in FILTERS:
        # Filter warnings before modifying for writing to `.env`
        filterwarnings(*filt)  # type: ignore  # pyright 1.1.317
        # Since `.env` files don't support regex matching, un-escape slashes and
        # truncate after the first `.*`.
        if len(filt) > (msg_pos := 1):
            msg = filt[msg_pos].replace("\\", "")  # type: ignore  # pyright 1.1.317
            for splittable in [",", ".*", "=", "'", '"']:
                msg = msg.split(splittable)[0]
            filt[msg_pos] = msg
        # Convert classes to string representations of their names
        if len(filt) > (warn_pos := 2):
            filt[warn_pos] = filt[warn_pos].__name__  # type: ignore  # pyright 1.1.317
        # Join filter parts with colons, as expected in `.env` files
        filters.append(":".join(str(f) for f in filt))
    # ! Truncate `.env` after notice and insert filters
    dotenv = Path(".env")
    content = dotenv.read_text(encoding="utf-8")
    notice = "# ! Don't edit below. `warning_filters.py` may trash any changes."
    loc = content.find(notice)  # Returns -1 if not found
    insert_point = len(content) if loc == -1 else loc
    content = f"{content[:insert_point]}{notice}\nPYTHONWARNINGS={','.join(filters)}\n"
    dotenv.write_text(encoding="utf-8", data=content)


FILTERS = (
    # Warn for all warnings, even `DeprecationWarning`
    ["default"],
    *(
        ["ignore", *w]
        for w in (
            # pkg_resources: https://github.com/blakeNaccarato/copier-python/issues/319#issue-1609228740
            (
                "Deprecated call to `pkg_resources.declare_namespace",
                DeprecationWarning,
            ),
            # Creating a LegacyVersion: https://github.com/blakeNaccarato/copier-python/issues/319#issuecomment-1454209165
            (
                "Creating a LegacyVersion has been deprecated and will be removed in the next major release",
                DeprecationWarning,
                "pip._vendor.packaging.version",
            ),
            # pre_commit read_text and open_text: https://github.com/blakeNaccarato/copier-python/issues/319#issuecomment-1455177114
            (
                "read_text is deprecated",
                DeprecationWarning,
                "pre_commit.util",
            ),
            (
                "open_text is deprecated",
                DeprecationWarning,
                "importlib.resources._legacy",
            ),
            # imp module and ABCs: https://github.com/blakeNaccarato/copier-python/issues/319#issuecomment-1455183241
            (
                "the imp module is deprecated in favour of importlib",
                DeprecationWarning,
                "googlecloudsdk.core.util.importing",
            ),
            (
                "Using or importing the ABCs",
                DeprecationWarning,
                "jsonschema.compat",
            ),
            # dpath...pkg_resources: https://github.com/blakeNaccarato/copier-python/issues/319#issuecomment-1457219723
            (
                "The dpath.util package is being deprecated.",
                DeprecationWarning,
                "dvc.dependency.param",
            ),
            # unclosed file...plumbum: https://github.com/blakeNaccarato/copier-python/issues/319#issuecomment-1458742412
            (
                "unclosed file",
                ResourceWarning,
                "dvc.stage.cache",
            ),
            (
                "the imp module is deprecated",
                DeprecationWarning,
                "ansiwrap.core",
            ),
            (
                "unclosed file",
                ResourceWarning,
                "ansiwrap.core",
            ),
            (
                "Jupyter is migrating its paths to use standard platformdirs",
                DeprecationWarning,
                "jupyter_client.connect",
            ),
            (
                "the file is not specified with any extension",
                UserWarning,
                "papermill.iorw",
            ),
            (
                "Passing unrecognized arguments to super",
                DeprecationWarning,
                "traitlets.config.configurable",
            ),
            (
                "pkg_resources is deprecated as an API",
                DeprecationWarning,
            ),
            (
                "unclosed file",
                ResourceWarning,
                "plumbum.commands.base",
            ),
            # ? NOT DOCUMENTED
            (
                "Passing unrecognized arguments to super",
                DeprecationWarning,
                "ipywidgets.widgets.widget",
            ),
            # ? NOT DOCUMENTED
            (
                "`ipykernel.pylab.backend_inline` is deprecated",
                DeprecationWarning,
                "ipykernel.pylab.backend_inline",
            ),
            # ? NOT DOCUMENTED
            (
                "subprocess",
                ResourceWarning,
                "subprocess",
            ),
            # BuiltinImporter.module_repr:DeprecationWarning:importlib._bootstrap: https://github.com/blakeNaccarato/copier-python/issues/319#issuecomment-1493542865
            (
                "BuiltinImporter.module_repr",
                DeprecationWarning,
                "importlib._bootstrap",
            ),
            # lib2to3: https://github.com/blakeNaccarato/copier-python/issues/319#issuecomment-1502112446
            (
                "lib2to3 package is deprecated and may not be able to parse Python",
                PendingDeprecationWarning,
            ),
            (
                "lib2to3 package is deprecated and may not be able to parse Python",
                DeprecationWarning,
            ),
            # ImportDenier:ImportWarning: https://github.com/blakeNaccarato/copier-python/issues/319#issuecomment-1546359005
            (
                "ImportDenier",
                ImportWarning,
            ),
            # numpy.ndarray size changed:RuntimeWarning: https://github.com/blakeNaccarato/copier-python/issues/319#issuecomment-1546360423
            (
                r"numpy.ndarray size changed",
                RuntimeWarning,
            ),
            # :DeprecationWarning:sphinx.util.images...sphinx_book_theme: https://github.com/blakeNaccarato/copier-python/issues/319#issuecomment-1591786172
            (
                "",
                DeprecationWarning,
                "sphinx.util.images",
            ),
            # ? NOT DOCUMENTED
            (
                "",
                DeprecationWarning,
                "myst_nb.sphinx_ext",
            ),
            # ? NOT DOCUMENTED
            (
                "",
                DeprecationWarning,
                "myst_parser.mdit_to_docutils.base",
            ),
            # ? NOT DOCUMENTED
            (
                "",
                DeprecationWarning,
                "optparse",
            ),
            # ? NOT DOCUMENTED
            (
                "Proactor event loop",
                RuntimeWarning,
                "zmq._future",
            ),
            # ? NOT DOCUMENTED
            (
                "",
                PendingDeprecationWarning,
                "myst_nb.ext.execution_tables",
            ),
            # ? NOT DOCUMENTED
            (
                "",
                PendingDeprecationWarning,
                "sphinx_book_theme",
            ),
            # trackpy: https://github.com/blakeNaccarato/copier-python/issues/319#issuecomment-1615198787
            (
                "Please use `binary_dilation`",
                DeprecationWarning,
                "trackpy.uncertainty",
            ),
            (
                "Importing clear_output from IPython.core.display is deprecated",
                DeprecationWarning,
                "trackpy.utils",
            ),
            # VSCode debugging: https://github.com/blakeNaccarato/copier-python/issues/319#issuecomment-1644830915
            (
                r"distutils Version classes are deprecated\. Use packaging\.version instead\.",
                DeprecationWarning,
            ),
            (
                r"unclosed file <_io\.BufferedReader name='.*'>",
                ResourceWarning,
            ),
            # ! NOT LINKED
            (
                (
                    "subpackages can technically be lazily loaded, but it causes the "
                    "package to be eagerly loaded even if it is already lazily loaded."
                    "So, you probably shouldn't use subpackages with this lazy feature."
                ),
                RuntimeWarning,
            ),
            (
                "invalid escape sequence",
                DeprecationWarning,
                "sparklines.sparklines",
            ),
        )
    ),
)

if __name__ == "__main__":
    filter_warnings_and_update_dotenv()



from typing import Mapping
from functools import partial

from ...classes.graph import Graph

__all__ = ["harmonic_centrality"]

def harmonic_centrality(
    G: Graph, nbunch=None, distance=None, sources=None
) -> Mapping: ...



from typing import Mapping
from numpy.typing import ArrayLike
from collections import defaultdict

from ..classes.graph import Graph

__all__ = ["check_planarity", "is_planar", "PlanarEmbedding"]

def is_planar(G: Graph) -> bool: ...
def check_planarity(G: Graph, counterexample: bool = False): ...
def check_planarity_recursive(G: Graph, counterexample=False): ...
def get_counterexample(G: Graph): ...
def get_counterexample_recursive(G: Graph): ...

class Interval:
    def __init__(self, low=None, high=None): ...
    def empty(self): ...
    def copy(self): ...
    def conflicting(self, b, planarity_state): ...

class ConflictPair:
    def __init__(self, left=..., right=...): ...
    def swap(self): ...
    def lowest(self, planarity_state): ...

def top_of_stack(l): ...

class LRPlanarity:

    __slots__: list = ...

    def __init__(self, G): ...
    def lr_planarity(self) -> Mapping: ...
    def lr_planarity_recursive(self): ...
    def dfs_orientation(self, v): ...
    def dfs_orientation_recursive(self, v): ...
    def dfs_testing(self, v): ...
    def dfs_testing_recursive(self, v): ...
    def add_constraints(self, ei, e): ...
    def remove_back_edges(self, e): ...
    def dfs_embedding(self, v): ...
    def dfs_embedding_recursive(self, v): ...
    def sign(self, e): ...
    def sign_recursive(self, e): ...

class PlanarEmbedding(nx.DiGraph):
    def get_data(self) -> Mapping: ...
    def set_data(self, data: Mapping): ...
    def neighbors_cw_order(self, v): ...
    def check_structure(self): ...
    def add_half_edge_ccw(self, start_node, end_node, reference_neighbor): ...
    def add_half_edge_cw(self, start_node, end_node, reference_neighbor): ...
    def connect_components(self, v, w): ...
    def add_half_edge_first(self, start_node, end_node): ...
    def next_face_half_edge(self, v, w) -> tuple: ...
    def traverse_face(self, v, w, mark_half_edges: set | None = None) -> ArrayLike: ...
    def is_directed(self): ...



from typing import Any

from django.contrib.gis.gdal.base import GDALBase as GDALBase

class DataSource(GDALBase):
    destructor: Any = ...
    encoding: Any = ...
    ptr: Any = ...
    driver: Any = ...
    def __init__(
        self,
        ds_input: Any,
        ds_driver: bool = ...,
        write: bool = ...,
        encoding: str = ...,
    ) -> None: ...
    def __getitem__(self, index: Any) -> Any: ...
    def __len__(self) -> Any: ...
    @property
    def layer_count(self) -> Any: ...
    @property
    def name(self) -> Any: ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.linalg._flinalg, version: $Revision: $
import typing
import builtins as _mod_builtins

__doc__: str
__file__: str
__name__: str
__package__: str
__version__: bytes
def cdet_c(a, overwrite_a=...) -> typing.Any:
    "det,info = cdet_c(a,[overwrite_a])\n\nWrapper for ``cdet_c``.\n\nParameters\n----------\na : input rank-2 array('F') with bounds (n,n)\n\nOther Parameters\n----------------\noverwrite_a : input int, optional\n    Default: 0\n\nReturns\n-------\ndet : complex\ninfo : int\n"
    ...

def cdet_r(a, overwrite_a=...) -> typing.Any:
    "det,info = cdet_r(a,[overwrite_a])\n\nWrapper for ``cdet_r``.\n\nParameters\n----------\na : input rank-2 array('F') with bounds (n,n)\n\nOther Parameters\n----------------\noverwrite_a : input int, optional\n    Default: 0\n\nReturns\n-------\ndet : complex\ninfo : int\n"
    ...

def clu_c(a, permute_l=..., overwrite_a=...) -> typing.Any:
    "p,l,u,info = clu_c(a,[permute_l,overwrite_a])\n\nWrapper for ``clu_c``.\n\nParameters\n----------\na : input rank-2 array('F') with bounds (m,n)\n\nOther Parameters\n----------------\noverwrite_a : input int, optional\n    Default: 0\npermute_l : input int, optional\n    Default: 0\n\nReturns\n-------\np : rank-2 array('f') with bounds (m1,m1)\nl : rank-2 array('F') with bounds (m,k)\nu : rank-2 array('F') with bounds (k,n)\ninfo : int\n"
    ...

def ddet_c(a, overwrite_a=...) -> typing.Any:
    "det,info = ddet_c(a,[overwrite_a])\n\nWrapper for ``ddet_c``.\n\nParameters\n----------\na : input rank-2 array('d') with bounds (n,n)\n\nOther Parameters\n----------------\noverwrite_a : input int, optional\n    Default: 0\n\nReturns\n-------\ndet : float\ninfo : int\n"
    ...

def ddet_r(a, overwrite_a=...) -> typing.Any:
    "det,info = ddet_r(a,[overwrite_a])\n\nWrapper for ``ddet_r``.\n\nParameters\n----------\na : input rank-2 array('d') with bounds (n,n)\n\nOther Parameters\n----------------\noverwrite_a : input int, optional\n    Default: 0\n\nReturns\n-------\ndet : float\ninfo : int\n"
    ...

def dlu_c(a, permute_l=..., overwrite_a=...) -> typing.Any:
    "p,l,u,info = dlu_c(a,[permute_l,overwrite_a])\n\nWrapper for ``dlu_c``.\n\nParameters\n----------\na : input rank-2 array('d') with bounds (m,n)\n\nOther Parameters\n----------------\noverwrite_a : input int, optional\n    Default: 0\npermute_l : input int, optional\n    Default: 0\n\nReturns\n-------\np : rank-2 array('d') with bounds (m1,m1)\nl : rank-2 array('d') with bounds (m,k)\nu : rank-2 array('d') with bounds (k,n)\ninfo : int\n"
    ...

def sdet_c(a, overwrite_a=...) -> typing.Any:
    "det,info = sdet_c(a,[overwrite_a])\n\nWrapper for ``sdet_c``.\n\nParameters\n----------\na : input rank-2 array('f') with bounds (n,n)\n\nOther Parameters\n----------------\noverwrite_a : input int, optional\n    Default: 0\n\nReturns\n-------\ndet : float\ninfo : int\n"
    ...

def sdet_r(a, overwrite_a=...) -> typing.Any:
    "det,info = sdet_r(a,[overwrite_a])\n\nWrapper for ``sdet_r``.\n\nParameters\n----------\na : input rank-2 array('f') with bounds (n,n)\n\nOther Parameters\n----------------\noverwrite_a : input int, optional\n    Default: 0\n\nReturns\n-------\ndet : float\ninfo : int\n"
    ...

def slu_c(a, permute_l=..., overwrite_a=...) -> typing.Any:
    "p,l,u,info = slu_c(a,[permute_l,overwrite_a])\n\nWrapper for ``slu_c``.\n\nParameters\n----------\na : input rank-2 array('f') with bounds (m,n)\n\nOther Parameters\n----------------\noverwrite_a : input int, optional\n    Default: 0\npermute_l : input int, optional\n    Default: 0\n\nReturns\n-------\np : rank-2 array('f') with bounds (m1,m1)\nl : rank-2 array('f') with bounds (m,k)\nu : rank-2 array('f') with bounds (k,n)\ninfo : int\n"
    ...

def zdet_c(a, overwrite_a=...) -> typing.Any:
    "det,info = zdet_c(a,[overwrite_a])\n\nWrapper for ``zdet_c``.\n\nParameters\n----------\na : input rank-2 array('D') with bounds (n,n)\n\nOther Parameters\n----------------\noverwrite_a : input int, optional\n    Default: 0\n\nReturns\n-------\ndet : complex\ninfo : int\n"
    ...

def zdet_r(a, overwrite_a=...) -> typing.Any:
    "det,info = zdet_r(a,[overwrite_a])\n\nWrapper for ``zdet_r``.\n\nParameters\n----------\na : input rank-2 array('D') with bounds (n,n)\n\nOther Parameters\n----------------\noverwrite_a : input int, optional\n    Default: 0\n\nReturns\n-------\ndet : complex\ninfo : int\n"
    ...

def zlu_c(a, permute_l=..., overwrite_a=...) -> typing.Any:
    "p,l,u,info = zlu_c(a,[permute_l,overwrite_a])\n\nWrapper for ``zlu_c``.\n\nParameters\n----------\na : input rank-2 array('D') with bounds (m,n)\n\nOther Parameters\n----------------\noverwrite_a : input int, optional\n    Default: 0\npermute_l : input int, optional\n    Default: 0\n\nReturns\n-------\np : rank-2 array('d') with bounds (m1,m1)\nl : rank-2 array('D') with bounds (m,k)\nu : rank-2 array('D') with bounds (k,n)\ninfo : int\n"
    ...

def __getattr__(name) -> typing.Any:
    ...




from typing import Iterator, Literal
from ._typing import Int
from contextlib import contextmanager as contextmanager
import os
import threading

_global_config: dict = ...
_threadlocal = ...


def get_config() -> dict | dict[str, bool | int | str]:
    ...


def set_config(
    assume_finite: None | bool = None,
    working_memory: None | Int = None,
    print_changed_only: None | bool = None,
    display: None | Literal["text", "diagram"] = None,
    pairwise_dist_chunk_size: None | Int = None,
    enable_cython_pairwise_dist: None | bool = None,
    array_api_dispatch: None | bool = None,
    transform_output: None | str = None,
) -> None:
    ...


def config_context(
    *,
    assume_finite: None | bool = None,
    working_memory: None | Int = None,
    print_changed_only: None | bool = None,
    display: None | Literal["text", "diagram"] = None,
    pairwise_dist_chunk_size: None | Int = None,
    enable_cython_pairwise_dist: None | bool = None,
    array_api_dispatch: None | bool = None,
    transform_output: None | str = None,
) -> Iterator[None]:
    ...



import tkinter as tk
from typing import Literal, Sequence
from matplotlib._api import classproperty
from matplotlib._typing import *
from matplotlib import backend_tools
from matplotlib.backend_bases import (
    FigureCanvasBase,
    FigureManagerBase,
    NavigationToolbar2,
    TimerBase,
    ToolContainerBase,
    _Backend,
)

backend_version: float = ...
cursord: dict[backend_tools.Cursors, str] = ...

TK_PHOTO_COMPOSITE_OVERLAY: Literal[0] = ...
TK_PHOTO_COMPOSITE_SET: Literal[1] = ...

def blit(photoimage, aggimage, offsets: Sequence[int], bbox=...)-> None: ...

class TimerTk(TimerBase):
    def __init__(self, parent, *args, **kwargs) -> None: ...

class FigureCanvasTk(FigureCanvasBase):
    required_interactive_framework: str = ...
    manager_class: classproperty = ...

    def __init__(self, figure=..., master=...) -> None: ...
    def resize(self, event)-> None: ...
    def draw_idle(self)-> None: ...
    def get_tk_widget(self)-> tk.Canvas: ...
    def motion_notify_event(self, event)-> None: ...
    def enter_notify_event(self, event)-> None: ...
    def button_press_event(self, event, dblclick=...)-> None: ...
    def button_dblclick_event(self, event)-> None: ...
    def button_release_event(self, event)-> None: ...
    def scroll_event(self, event) -> None: ...
    def scroll_event_windows(self, event)-> None: ...
    def key_press(self, event)-> None: ...
    def key_release(self, event)-> None: ...
    def new_timer(self, *args, **kwargs)-> TimerTk: ...
    def flush_events(self)-> None: ...
    def start_event_loop(self, timeout: int=0)-> None: ...
    def stop_event_loop(self)-> None: ...
    def set_cursor(self, cursor)-> None: ...

class FigureManagerTk(FigureManagerBase):

    canvas: FigureCanvasBase
    num: int | str
    toolbar: tk.Toolbar
    window: tk.Window

    def __init__(self, canvas, num, window) -> None: ...
    @classmethod
    def create_with_canvas(cls, canvas_class, figure, num)-> FigureManagerTk: ...
    def resize(self, width: float, height: float) -> None: ...
    def show(self)-> None: ...
    def destroy(self, *args)-> None: ...
    def get_window_title(self): ...
    def set_window_title(self, title)-> None: ...
    def full_screen_toggle(self)-> None: ...

class NavigationToolbar2Tk(NavigationToolbar2, tk.Frame):
    window: tk.Window = ...
    def __init__(
        self, canvas: FigureCanvasBase, window=..., *, pack_toolbar: bool = True
    ) -> None: ...
    def pan(self, *args)-> None: ...
    def zoom(self, *args)-> None: ...
    def set_message(self, s: str)-> None: ...
    def draw_rubberband(self, event, x0: float, y0: float, x1: float, y1: float)-> None: ...
    def remove_rubberband(self)-> None: ...

    def save_figure(self, *args)-> None: ...
    def set_history_buttons(self)-> None: ...

class ToolTip:
    @staticmethod
    def createToolTip(widget, text)-> None: ...
    def __init__(self, widget) -> None: ...
    def showtip(self, text)-> None: ...
    def hidetip(self)-> None: ...

class RubberbandTk(backend_tools.RubberbandBase):
    def draw_rubberband(self, x0, y0, x1, y1)-> None: ...
    def remove_rubberband(self)-> None: ...

class SetCursorTk(backend_tools.SetCursorBase):
    def set_cursor(self, cursor)-> None: ...

class ToolbarTk(ToolContainerBase, tk.Frame):
    def __init__(self, toolmanager, window=...) -> None: ...
    def add_toolitem(self, name, group, position, image_file, description, toggle)-> None: ...
    def toggle_toolitem(self, name, toggled)-> None: ...
    def remove_toolitem(self, name)-> None: ...
    def set_message(self, s)-> None: ...

class SaveFigureTk(backend_tools.SaveFigureBase):
    def trigger(self, *args)-> None: ...

class ConfigureSubplotsTk(backend_tools.ConfigureSubplotsBase):
    def trigger(self, *args)-> None: ...

class HelpTk(backend_tools.ToolHelpBase):
    def trigger(self, *args)-> None: ...

Toolbar = ToolbarTk

class _BackendTk(_Backend):
    FigureManager = FigureManagerTk
    @staticmethod
    def mainloop()-> None: ...



"""Bump `pyproject.toml` and `pre-commit.config` with dependency changes."""

from pathlib import Path

import re
from pathlib import Path

import toml

# Bump `pyproject.toml` based on changes in `requirements.txt`
dependencies = [
    line.rstrip().replace("==", ">=")
    for line in Path("requirements.txt").read_text(encoding="utf-8").splitlines()
    if line and not line.startswith("#")
]
source = toml.loads((".tools" / Path("pyproject.toml")).read_text(encoding="utf-8"))
source["project"]["dependencies"] = dependencies
Path("pyproject.toml").write_text(encoding="utf-8", data=toml.dumps(source))



from io import BytesIO
from matplotlib._typing import *
from matplotlib.backend_bases import RendererBase
from matplotlib.font_manager import FontProperties

def get_glyphs_subset(fontfile: str, characters: str) -> BytesIO: ...

class CharacterTracker:
    def __init__(self) -> None: ...
    def track(self, font, s: str)-> None: ...
    def track_glyph(self, font, glyph)-> None: ...

class RendererPDFPSBase(RendererBase):
    def __init__(self, width, height) -> None: ...
    def flipy(self) -> bool: ...
    def option_scale_image(self)-> bool: ...
    def option_image_nocomposite(self) -> bool: ...
    def get_canvas_width_height(self)-> tuple[float, float]: ...
    def get_text_width_height_descent(
        self, s: str, prop: FontProperties, ismath: bool
    ) -> tuple[float, float, float]: ...



from typing import Callable
from .figure import Figure
from typing import Callable

def adjust_bbox(fig: Figure, bbox_inches, fixed_dpi: float|None=None)-> Callable[[],None]: ...
def process_figure_for_rasterizing(fig: Figure, bbox_inches_restore, fixed_dpi: float|None=None)-> tuple[float, Callable[[], None]]: ...



from typing import (
    Callable,
    Iterable,
    Mapping,
    Sequence,
    Union,
)

EscapeChars = Union[Mapping[str, str], Iterable[str]]

def adjoin(space: int, *lists: list[str], **kwargs) -> str: ...
def justify(texts: Iterable[str], max_len: int, mode: str = ...) -> list[str]: ...
def pprint_thing(
    thing,
    _nest_lvl: int = ...,
    escape_chars: EscapeChars | None = ...,
    default_escapes: bool = ...,
    quote_strings: bool = ...,
    max_seq_items: int | None = ...,
) -> str: ...
def pprint_thing_encoded(object, encoding: str = ..., errors: str = ...) -> bytes: ...

default_pprint = ...

def format_object_summary(
    obj,
    formatter: Callable,
    is_justify: bool = ...,
    name: str | None = ...,
    indent_for_name: bool = ...,
    line_break_each_value: bool = ...,
) -> str: ...
def format_object_attrs(
    obj: Sequence, include_dtype: bool = ...
) -> list[tuple[str, str | int]]: ...



from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------
import numpy as np

from .visual import Visual
from .. import gloo

class ScrollingLinesVisual(Visual):

    vertex_code: str = ...

    fragment_code: str = ...

    def __init__(
        self,
        n_lines: int,
        line_size: int,
        dx: float,
        color: ArrayLike | None = None,
        pos_offset: ArrayLike | None = None,
        columns: int | None = None,
        cell_size: tuple | None = None,
    ): ...
    def set_pos_offset(self, po: ArrayLike): ...
    def set_color(self, color: ArrayLike): ...
    def _prepare_transforms(self, view): ...
    def _prepare_draw(self, view): ...
    def _compute_bounds(self, axis, view): ...
    def roll_data(self, data: ArrayLike): ...
    def set_data(self, index: int, data: ArrayLike): ...



import sys
import warnings

import pytest

import networkx

def pytest_addoption(parser): ...
def pytest_configure(config): ...
def pytest_collection_modifyitems(config, items): ...

# TODO: The warnings below need to be dealt with, but for now we silence them.
@pytest.fixture(autouse=True)
def set_warnings(): ...
@pytest.fixture(autouse=True)
def add_nx(doctest_namespace): ...

# List of files that pytest should ignore

collect_ignore: list = ...

needs_numpy: list = ...
needs_scipy: list = ...
needs_matplotlib: list = ...
needs_pandas: list = ...
needs_yaml: list = ...
needs_pygraphviz: list = ...
needs_pydot: list = ...
needs_ogr: list = ...
needs_sympy: list = ...



from typing import ClassVar, Literal
from numpy.random import RandomState
from scipy.special import betaln as betaln, digamma as digamma, gammaln as gammaln
from ._base import BaseMixture
from numpy import ndarray
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from numbers import Real as Real
from .._typing import Int, Float, ArrayLike
from ..utils import check_array as check_array

# Author: Wei Xue <xuewei4d@gmail.com>
#         Thierry Guillemot <thierry.guillemot.work@gmail.com>
# License: BSD 3 clause

import math
import numpy as np


class BayesianGaussianMixture(BaseMixture):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    covariance_prior_: float | ArrayLike = ...
    degrees_of_freedom_: ArrayLike = ...
    degrees_of_freedom_prior_: float = ...
    mean_prior_: ArrayLike = ...
    mean_precision_: ArrayLike = ...
    mean_precision_prior_: float = ...
    weight_concentration_: ArrayLike = ...
    weight_concentration_prior_: float | tuple = ...
    lower_bound_: float = ...
    n_iter_: int = ...
    converged_: bool = ...
    precisions_cholesky_: ArrayLike = ...
    precisions_: ArrayLike = ...
    covariances_: ArrayLike = ...
    means_: ArrayLike = ...
    weights_: ArrayLike = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        n_components: Int = 1,
        covariance_type: Literal["full", "tied", "diag", "spherical", "full"] = "full",
        tol: Float = 1e-3,
        reg_covar: Float = 1e-6,
        max_iter: Int = 100,
        n_init: Int = 1,
        init_params: Literal[
            "kmeans", "k-means++", "random", "random_from_data", "kmeans"
        ] = "kmeans",
        weight_concentration_prior_type: Literal[
            "dirichlet_process", "dirichlet_distribution", "dirichlet_process"
        ] = "dirichlet_process",
        weight_concentration_prior: None | Float = None,
        mean_precision_prior: None | Float = None,
        mean_prior: None | ArrayLike = None,
        degrees_of_freedom_prior: None | Float = None,
        covariance_prior: float | None | ArrayLike = None,
        random_state: RandomState | None | Int = None,
        warm_start: bool = False,
        verbose: Int = 0,
        verbose_interval: Int = 10,
    ) -> None:
        ...



"""
This type stub file was generated by pyright.
"""

from contextlib import contextmanager
from io import StringIO
from IPython.utils.decorators import undoc

"""
Python advanced pretty printer.  This pretty printer is intended to
replace the old `pprint` python module which does not allow developers
to provide their own pretty print callbacks.

This module is based on ruby's `prettyprint.rb` library by `Tanaka Akira`.


Example Usage
-------------

To directly print the representation of an object use `pprint`::

    from pretty import pprint
    pprint(complex_object)

To get a string of the output use `pretty`::

    from pretty import pretty
    string = pretty(complex_object)


Extending
---------

The pretty library allows developers to add pretty printing rules for their
own objects.  This process is straightforward.  All you have to do is to
add a `_repr_pretty_` method to your object and call the methods on the
pretty printer passed::

    class MyObject(object):

        def _repr_pretty_(self, p, cycle):
            ...

Here's an example for a class with a simple constructor::

    class MySimpleObject:

        def __init__(self, a, b, *, c=None):
            self.a = a
            self.b = b
            self.c = c

        def _repr_pretty_(self, p, cycle):
            ctor = CallExpression.factory(self.__class__.__name__)
            if self.c is None:
                p.pretty(ctor(a, b))
            else:
                p.pretty(ctor(a, b, c=c))

Here is an example implementation of a `_repr_pretty_` method for a list
subclass::

    class MyList(list):

        def _repr_pretty_(self, p, cycle):
            if cycle:
                p.text('MyList(...)')
            else:
                with p.group(8, 'MyList([', '])'):
                    for idx, item in enumerate(self):
                        if idx:
                            p.text(',')
                            p.breakable()
                        p.pretty(item)

The `cycle` parameter is `True` if pretty detected a cycle.  You *have* to
react to that or the result is an infinite loop.  `p.text()` just adds
non breaking text to the output, `p.breakable()` either adds a whitespace
or breaks here.  If you pass it an argument it's used instead of the
default space.  `p.pretty` prettyprints another object using the pretty print
method.

The first parameter to the `group` function specifies the extra indentation
of the next line.  In this example the next item will either be on the same
line (if the items are short enough) or aligned with the right edge of the
opening bracket of `MyList`.

If you just want to indent something you can use the group function
without open / close parameters.  You can also use this code::

    with p.indent(2):
        ...

Inheritance diagram:

.. inheritance-diagram:: IPython.lib.pretty
   :parts: 3

:copyright: 2007 by Armin Ronacher.
            Portions (c) 2009 by Robert Kern.
:license: BSD License.
"""
__all__ = ['pretty', 'pprint', 'PrettyPrinter', 'RepresentationPrinter', 'for_type', 'for_type_by_name', 'RawText', 'RawStringLiteral', 'CallExpression']
MAX_SEQ_LENGTH = ...
_re_pattern_type = ...
@undoc
class CUnicodeIO(StringIO):
    def __init__(self, *args, **kwargs) -> None:
        ...
    


def pretty(obj, verbose=..., max_width=..., newline=..., max_seq_length=...):
    """
    Pretty print the object's representation.
    """
    ...

def pprint(obj, verbose=..., max_width=..., newline=..., max_seq_length=...): # -> None:
    """
    Like `pretty` but print to stdout.
    """
    ...

class _PrettyPrinterBase:
    @contextmanager
    def indent(self, indent):
        """with statement support for indenting/dedenting."""
        ...
    
    @contextmanager
    def group(self, indent=..., open=..., close=...):
        """like begin_group / end_group but for the with statement."""
        ...
    


class PrettyPrinter(_PrettyPrinterBase):
    """
    Baseclass for the `RepresentationPrinter` prettyprinter that is used to
    generate pretty reprs of objects.  Contrary to the `RepresentationPrinter`
    this printer knows nothing about the default pprinters or the `_repr_pretty_`
    callback method.
    """
    def __init__(self, output, max_width=..., newline=..., max_seq_length=...) -> None:
        ...
    
    def text(self, obj): # -> None:
        """Add literal text to the output."""
        ...
    
    def breakable(self, sep=...): # -> None:
        """
        Add a breakable separator to the output.  This does not mean that it
        will automatically break here.  If no breaking on this position takes
        place the `sep` is inserted which default to one space.
        """
        ...
    
    def break_(self): # -> None:
        """
        Explicitly insert a newline into the output, maintaining correct indentation.
        """
        ...
    
    def begin_group(self, indent=..., open=...): # -> None:
        """
        Begin a group.
        The first parameter specifies the indentation for the next line (usually
        the width of the opening text), the second the opening text.  All
        parameters are optional.
        """
        ...
    
    def end_group(self, dedent=..., close=...): # -> None:
        """End a group. See `begin_group` for more details."""
        ...
    
    def flush(self): # -> None:
        """Flush data that is left in the buffer."""
        ...
    


class RepresentationPrinter(PrettyPrinter):
    """
    Special pretty printer that has a `pretty` method that calls the pretty
    printer for a python object.

    This class stores processing data on `self` so you must *never* use
    this class in a threaded environment.  Always lock it or reinstanciate
    it.

    Instances also have a verbose flag callbacks can access to control their
    output.  For example the default instance repr prints all attributes and
    methods that are not prefixed by an underscore if the printer is in
    verbose mode.
    """
    def __init__(self, output, verbose=..., max_width=..., newline=..., singleton_pprinters=..., type_pprinters=..., deferred_pprinters=..., max_seq_length=...) -> None:
        ...
    
    def pretty(self, obj): # -> None:
        """Pretty print the given object."""
        ...
    


class Printable:
    def output(self, stream, output_width):
        ...
    


class Text(Printable):
    def __init__(self) -> None:
        ...
    
    def output(self, stream, output_width):
        ...
    
    def add(self, obj, width): # -> None:
        ...
    


class Breakable(Printable):
    def __init__(self, seq, width, pretty) -> None:
        ...
    
    def output(self, stream, output_width):
        ...
    


class Group(Printable):
    def __init__(self, depth) -> None:
        ...
    


class GroupQueue:
    def __init__(self, *groups) -> None:
        ...
    
    def enq(self, group): # -> None:
        ...
    
    def deq(self): # -> None:
        ...
    
    def remove(self, group): # -> None:
        ...
    


class RawText:
    """ Object such that ``p.pretty(RawText(value))`` is the same as ``p.text(value)``.

    An example usage of this would be to show a list as binary numbers, using
    ``p.pretty([RawText(bin(i)) for i in integers])``.
    """
    def __init__(self, value) -> None:
        ...
    


class CallExpression:
    """ Object which emits a line-wrapped call expression in the form `__name(*args, **kwargs)` """
    def __init__(__self, __name, *args, **kwargs) -> None:
        ...
    
    @classmethod
    def factory(cls, name): # -> (*args: Unknown, **kwargs: Unknown) -> Unknown:
        ...
    


class RawStringLiteral:
    """ Wrapper that shows a string with a `r` prefix """
    def __init__(self, value) -> None:
        ...
    


class _ReFlags:
    def __init__(self, value) -> None:
        ...
    


_exception_base = ...
_type_pprinters = ...
_env_type = ...
if _env_type is not dict:
    ...
_deferred_type_pprinters = ...
def for_type(typ, func):
    """
    Add a pretty printer for a given type.
    """
    ...

def for_type_by_name(type_module, type_name, func):
    """
    Add a pretty printer for a type specified by the module and name of a type
    rather than the type object itself.
    """
    ...

_singleton_pprinters = ...
if __name__ == '__main__':
    class Foo:
        def __init__(self) -> None:
            ...
        
        def get_foo(self): # -> None:
            ...
        
    
    



from . import graph as graph
from .manual_segmentation import (
    manual_polygon_segmentation as manual_polygon_segmentation,
)
from .manual_segmentation import manual_lasso_segmentation as manual_lasso_segmentation
from .trainable_segmentation import (
    fit_segmenter as fit_segmenter,
    predict_segmenter as predict_segmenter,
    TrainableSegmenter as TrainableSegmenter,
)

__all__ = [
    "graph",
    "manual_lasso_segmentation",
    "manual_polygon_segmentation",
    "fit_segmenter",
    "predict_segmenter",
    "TrainableSegmenter",
]



from ._column_transformer import (
    ColumnTransformer as ColumnTransformer,
    make_column_transformer as make_column_transformer,
    make_column_selector as make_column_selector,
)
from ._target import TransformedTargetRegressor as TransformedTargetRegressor


__all__ = [
    "ColumnTransformer",
    "make_column_transformer",
    "TransformedTargetRegressor",
    "make_column_selector",
]



from vispy.util.svg.color import Color
from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import numpy as np
from .polygon import PolygonVisual

class EllipseVisual(PolygonVisual):
    def __init__(
        self,
        center: ArrayLike | None = None,
        color: Color | str = "black",
        border_color: Color | None = None,
        border_width: float = 1,
        radius: float | tuple = ...,
        start_angle: float = 0.0,
        span_angle: float = 360.0,
        num_segments: int = 100,
        **kwargs,
    ): ...
    @staticmethod
    def _generate_vertices(center, radius, start_angle, span_angle, num_segments): ...
    @property
    def center(self): ...
    @center.setter
    def center(self, center): ...
    @property
    def radius(self): ...
    @radius.setter
    def radius(self, radius): ...
    @property
    def start_angle(self): ...
    @start_angle.setter
    def start_angle(self, start_angle): ...
    @property
    def span_angle(self): ...
    @span_angle.setter
    def span_angle(self, span_angle): ...
    @property
    def num_segments(self): ...
    @num_segments.setter
    def num_segments(self, num_segments): ...
    def _regen_pos(self): ...



import itertools

from ..classes.graph import Graph

__all__ = ["margulis_gabber_galil_graph", "chordal_cycle_graph", "paley_graph"]

# Other discrete torus expanders can be constructed by using the following edge
# sets. For more information, see Chapter 4, "Expander Graphs", in
# "Pseudorandomness", by Salil Vadhan.
#
# For a directed expander, add edges from (x, y) to:
#
#     (x, y),
#     ((x + 1) % n, y),
#     (x, (y + 1) % n),
#     (x, (x + y) % n),
#     (-y % n, x)
#
# For an undirected expander, add the reverse edges.
#
# Also appearing in the paper of Gabber and Galil:
#
#     (x, y),
#     (x, (x + y) % n),
#     (x, (x + y + 1) % n),
#     ((x + y) % n, y),
#     ((x + y + 1) % n, y)
#
# and:
#
#     (x, y),
#     ((x + 2*y) % n, y),
#     ((x + (2*y + 1)) % n, y),
#     ((x + (2*y + 2)) % n, y),
#     (x, (y + 2*x) % n),
#     (x, (y + (2*x + 1)) % n),
#     (x, (y + (2*x + 2)) % n),
#
def margulis_gabber_galil_graph(n: int, create_using=None): ...
def chordal_cycle_graph(p, create_using=None): ...
def paley_graph(p, create_using=None): ...



from typing import (
    Callable,
    ClassVar,
    Iterable,
    Iterator,
    Literal,
    Mapping,
    Sequence,
    TypeVar,
)
from numpy.random import RandomState
from ..base import BaseEstimator
from ._split import BaseShuffleSplit
from ._split import check_cv as check_cv
from copy import deepcopy as deepcopy
from ..svm._classes import SVC
from abc import abstractmethod as abstractmethod
from numpy import ndarray
from numbers import Integral as Integral
from ..base import is_classifier as is_classifier
from ..utils import resample as resample
from . import ParameterGrid as ParameterGrid, ParameterSampler as ParameterSampler
from ..utils.multiclass import (
    check_classification_targets as check_classification_targets,
)
from math import ceil as ceil, floor as floor, log as log
from ._search import BaseSearchCV
from .._typing import MatrixLike, ArrayLike, Int, Float
from . import BaseCrossValidator

BaseSuccessiveHalving_Self = TypeVar(
    "BaseSuccessiveHalving_Self", bound="BaseSuccessiveHalving"
)


import numpy as np


__all__ = ["HalvingGridSearchCV", "HalvingRandomSearchCV"]


class _SubsampleMetaSplitter:
    def __init__(self, *, base_cv, fraction, subsample_test, random_state) -> None:
        ...

    def split(
        self, X: ndarray, y: ndarray, groups=None
    ) -> Iterator[tuple[ndarray, ndarray]]:
        ...


class BaseSuccessiveHalving(BaseSearchCV):
    def __init__(
        self,
        estimator,
        *,
        scoring=None,
        n_jobs=None,
        refit: bool = True,
        cv: int = 5,
        verbose: int = 0,
        random_state=None,
        error_score=...,
        return_train_score: bool = True,
        max_resources: str = "auto",
        min_resources: str = "exhaust",
        resource: str = "n_samples",
        factor: int = 3,
        aggressive_elimination: bool = False,
    ) -> None:
        ...

    def fit(
        self: BaseSuccessiveHalving_Self,
        X: MatrixLike,
        y: None | MatrixLike | ArrayLike = None,
        groups: None | ArrayLike = None,
        **fit_params,
    ) -> BaseSuccessiveHalving_Self:
        ...


class HalvingGridSearchCV(BaseSuccessiveHalving):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    classes_: ndarray = ...
    multimetric_: bool = ...
    refit_time_: float = ...
    n_splits_: int = ...
    scorer_: Callable | dict = ...
    best_index_: int = ...
    best_params_: dict = ...
    best_score_: float = ...
    best_estimator_: BaseEstimator | dict = ...
    cv_results_: dict[str, ndarray] = ...
    n_required_iterations_: int = ...
    n_possible_iterations_: int = ...
    n_iterations_: int = ...
    min_resources_: int = ...
    max_resources_: int = ...
    n_remaining_candidates_: int = ...
    n_candidates_: list[int] = ...
    n_resources_: list[int] = ...

    _required_parameters: ClassVar[list] = ...

    def __init__(
        self,
        estimator: BaseEstimator | SVC,
        param_grid: Mapping | dict[str, list[int | float]] | Sequence[dict],
        *,
        factor: float | int = 3,
        resource: str = "n_samples",
        max_resources: str | Int = "auto",
        min_resources: int | Literal["exhaust", "smallest", "exhaust"] = "exhaust",
        aggressive_elimination: bool = False,
        cv: Iterable | int | BaseShuffleSplit | BaseCrossValidator = 5,
        scoring: None | str | Callable = None,
        refit: bool = True,
        error_score: str | Float = ...,
        return_train_score: bool = True,
        random_state: RandomState | None | Int = None,
        n_jobs: None | int = None,
        verbose: Int = 0,
    ) -> None:
        ...


class HalvingRandomSearchCV(BaseSuccessiveHalving):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    classes_: ndarray = ...
    multimetric_: bool = ...
    refit_time_: float = ...
    n_splits_: int = ...
    scorer_: Callable | dict = ...
    best_index_: int = ...
    best_params_: dict = ...
    best_score_: float = ...
    best_estimator_: BaseEstimator | dict = ...
    cv_results_: dict[str, ndarray] = ...
    n_required_iterations_: int = ...
    n_possible_iterations_: int = ...
    n_iterations_: int = ...
    min_resources_: int = ...
    max_resources_: int = ...
    n_remaining_candidates_: int = ...
    n_candidates_: list[int] = ...
    n_resources_: list[int] = ...

    _required_parameters: ClassVar[list] = ...

    def __init__(
        self,
        estimator: BaseEstimator,
        param_distributions: dict,
        *,
        n_candidates: str | Int = "exhaust",
        factor: float | int = 3,
        resource: str = "n_samples",
        max_resources: str | Int = "auto",
        min_resources: Literal["exhaust", "smallest", "smallest"] | int = "smallest",
        aggressive_elimination: bool = False,
        cv: Iterable | int | BaseShuffleSplit | BaseCrossValidator = 5,
        scoring: None | str | Callable = None,
        refit: bool = True,
        error_score: str | Float = ...,
        return_train_score: bool = True,
        random_state: RandomState | None | Int = None,
        n_jobs: None | int = None,
        verbose: Int = 0,
    ) -> None:
        ...



"""
This type stub file was generated by pyright.
"""

"""Compatibility tricks for Python 3. Mainly to do with unicode.

This file is deprecated and will be removed in a future version.
"""
def decode(s, encoding=...):
    ...

def encode(u, encoding=...):
    ...

def cast_unicode(s, encoding=...):
    ...

def safe_unicode(e):
    """unicode(e) with various fallbacks. Used for exceptions, which may not be
    safe to call unicode() on.
    """
    ...

def input(prompt=...):
    ...

def execfile(fname, glob, loc=..., compiler=...): # -> None:
    ...

PYPY = ...
def no_code(x, encoding=...):
    ...

cast_bytes_py2 = ...



"""
This type stub file was generated by pyright.
"""

from IPython.core.magic import Magics, line_magic, magics_class
from IPython.testing.skipdoctest import skip_doctest

"""Implementation of code management magic functions.
"""
class MacroToEdit(ValueError):
    ...


ipython_input_pat = ...
range_re = ...
def extract_code_ranges(ranges_str):
    """Turn a string of range for %%load into 2-tuples of (start, stop)
    ready to use as a slice of the content split by lines.

    Examples
    --------
    list(extract_input_ranges("5-10 2"))
    [(4, 10), (1, 2)]
    """
    ...

def extract_symbols(code, symbols):
    """
    Return a tuple  (blocks, not_found)
    where ``blocks`` is a list of code fragments
    for each symbol parsed from code, and ``not_found`` are
    symbols not found in the code.

    For example::

        In [1]: code = '''a = 10
           ...: def b(): return 42
           ...: class A: pass'''

        In [2]: extract_symbols(code, 'A,b,z')
        Out[2]: (['class A: pass\\n', 'def b(): return 42\\n'], ['z'])
    """
    ...

def strip_initial_indent(lines):
    """For %load, strip indent from lines until finding an unindented line.

    https://github.com/ipython/ipython/issues/9775
    """
    ...

class InteractivelyDefined(Exception):
    """Exception for interactively defined variable in magic_edit"""
    def __init__(self, index) -> None:
        ...
    


@magics_class
class CodeMagics(Magics):
    """Magics related to code management (loading, saving, editing, ...)."""
    def __init__(self, *args, **kwargs) -> None:
        ...
    
    @line_magic
    def save(self, parameter_s=...): # -> None:
        """Save a set of lines or a macro to a given filename.

        Usage:\\
          %save [options] filename [history]

        Options:

          -r: use 'raw' input.  By default, the 'processed' history is used,
          so that magics are loaded in their transformed version to valid
          Python.  If this option is given, the raw input as typed as the
          command line is used instead.
          
          -f: force overwrite.  If file exists, %save will prompt for overwrite
          unless -f is given.

          -a: append to the file instead of overwriting it.

        The history argument uses the same syntax as %history for input ranges,
        then saves the lines to the filename you specify.

        If no ranges are specified, saves history of the current session up to
        this point.

        It adds a '.py' extension to the file if you don't do so yourself, and
        it asks for confirmation before overwriting existing files.

        If `-r` option is used, the default extension is `.ipy`.
        """
        ...
    
    @line_magic
    def pastebin(self, parameter_s=...): # -> None:
        """Upload code to dpaste.com, returning the URL.

        Usage:\\
          %pastebin [-d "Custom description"][-e 24] 1-7

        The argument can be an input history range, a filename, or the name of a
        string or macro.

        If no arguments are given, uploads the history of this session up to
        this point.

        Options:

          -d: Pass a custom description. The default will say
              "Pasted from IPython".
          -e: Pass number of days for the link to be expired.
              The default will be 7 days.
        """
        ...
    
    @line_magic
    def loadpy(self, arg_s): # -> None:
        """Alias of `%load`

        `%loadpy` has gained some flexibility and dropped the requirement of a `.py`
        extension. So it has been renamed simply into %load. You can look at
        `%load`'s docstring for more info.
        """
        ...
    
    @line_magic
    def load(self, arg_s): # -> None:
        """Load code into the current frontend.

        Usage:\\
          %load [options] source

          where source can be a filename, URL, input history range, macro, or
          element in the user namespace

        If no arguments are given, loads the history of this session up to this
        point.

        Options:

          -r <lines>: Specify lines or ranges of lines to load from the source.
          Ranges could be specified as x-y (x..y) or in python-style x:y 
          (x..(y-1)). Both limits x and y can be left blank (meaning the 
          beginning and end of the file, respectively).

          -s <symbols>: Specify function or classes to load from python source. 

          -y : Don't ask confirmation for loading source above 200 000 characters.

          -n : Include the user's namespace when searching for source code.

        This magic command can either take a local filename, a URL, an history
        range (see %history) or a macro as argument, it will prompt for
        confirmation before loading source with more than 200 000 characters, unless
        -y flag is passed or if the frontend does not support raw_input::

        %load
        %load myscript.py
        %load 7-27
        %load myMacro
        %load http://www.example.com/myscript.py
        %load -r 5-10 myscript.py
        %load -r 10-20,30,40: foo.py
        %load -s MyClass,wonder_function myscript.py
        %load -n MyClass
        %load -n my_module.wonder_function
        """
        ...
    
    @skip_doctest
    @line_magic
    def edit(self, parameter_s=..., last_call=...): # -> None:
        """Bring up an editor and execute the resulting code.

        Usage:
          %edit [options] [args]

        %edit runs IPython's editor hook. The default version of this hook is
        set to call the editor specified by your $EDITOR environment variable.
        If this isn't found, it will default to vi under Linux/Unix and to
        notepad under Windows. See the end of this docstring for how to change
        the editor hook.

        You can also set the value of this editor via the
        ``TerminalInteractiveShell.editor`` option in your configuration file.
        This is useful if you wish to use a different editor from your typical
        default with IPython (and for Windows users who typically don't set
        environment variables).

        This command allows you to conveniently edit multi-line code right in
        your IPython session.

        If called without arguments, %edit opens up an empty editor with a
        temporary file and will execute the contents of this file when you
        close it (don't forget to save it!).


        Options:

        -n <number>: open the editor at a specified line number.  By default,
        the IPython editor hook uses the unix syntax 'editor +N filename', but
        you can configure this by providing your own modified hook if your
        favorite editor supports line-number specifications with a different
        syntax.

        -p: this will call the editor with the same data as the previous time
        it was used, regardless of how long ago (in your current session) it
        was.

        -r: use 'raw' input.  This option only applies to input taken from the
        user's history.  By default, the 'processed' history is used, so that
        magics are loaded in their transformed version to valid Python.  If
        this option is given, the raw input as typed as the command line is
        used instead.  When you exit the editor, it will be executed by
        IPython's own processor.

        -x: do not execute the edited code immediately upon exit. This is
        mainly useful if you are editing programs which need to be called with
        command line arguments, which you can then do using %run.


        Arguments:

        If arguments are given, the following possibilities exist:

        - If the argument is a filename, IPython will load that into the
          editor. It will execute its contents with execfile() when you exit,
          loading any code in the file into your interactive namespace.

        - The arguments are ranges of input history,  e.g. "7 ~1/4-6".
          The syntax is the same as in the %history magic.

        - If the argument is a string variable, its contents are loaded
          into the editor. You can thus edit any string which contains
          python code (including the result of previous edits).

        - If the argument is the name of an object (other than a string),
          IPython will try to locate the file where it was defined and open the
          editor at the point where it is defined. You can use `%edit function`
          to load an editor exactly at the point where 'function' is defined,
          edit it and have the file be executed automatically.

        - If the object is a macro (see %macro for details), this opens up your
          specified editor with a temporary file containing the macro's data.
          Upon exit, the macro is reloaded with the contents of the file.

        Note: opening at an exact line is only supported under Unix, and some
        editors (like kedit and gedit up to Gnome 2.8) do not understand the
        '+NUMBER' parameter necessary for this feature. Good editors like
        (X)Emacs, vi, jed, pico and joe all do.

        After executing your code, %edit will return as output the code you
        typed in the editor (except when it was an existing file). This way
        you can reload the code in further invocations of %edit as a variable,
        via _<NUMBER> or Out[<NUMBER>], where <NUMBER> is the prompt number of
        the output.

        Note that %edit is also available through the alias %ed.

        This is an example of creating a simple function inside the editor and
        then modifying it. First, start up the editor::

          In [1]: edit
          Editing... done. Executing edited code...
          Out[1]: 'def foo():\\n    print "foo() was defined in an editing
          session"\\n'

        We can then call the function foo()::

          In [2]: foo()
          foo() was defined in an editing session

        Now we edit foo.  IPython automatically loads the editor with the
        (temporary) file where foo() was previously defined::

          In [3]: edit foo
          Editing... done. Executing edited code...

        And if we call foo() again we get the modified version::

          In [4]: foo()
          foo() has now been changed!

        Here is an example of how to edit a code snippet successive
        times. First we call the editor::

          In [5]: edit
          Editing... done. Executing edited code...
          hello
          Out[5]: "print 'hello'\\n"

        Now we call it again with the previous output (stored in _)::

          In [6]: edit _
          Editing... done. Executing edited code...
          hello world
          Out[6]: "print 'hello world'\\n"

        Now we call it with the output #8 (stored in _8, also as Out[8])::

          In [7]: edit _8
          Editing... done. Executing edited code...
          hello again
          Out[7]: "print 'hello again'\\n"


        Changing the default editor hook:

        If you wish to write your own editor hook, you can put it in a
        configuration file which you load at startup time.  The default hook
        is defined in the IPython.core.hooks module, and you can use that as a
        starting example for further modifications.  That file also has
        general instructions on how to set a new hook for use once you've
        defined it."""
        ...
    





from typing import Literal, Sequence, Type
from libcst import For
from .backend_bases import RendererBase
from .gridspec import SubplotSpec
from .text import Text
from .cm import ScalarMappable
from .colors import Colormap, Normalize
from .collections import LineCollection
from .ticker import Formatter, Locator
from .axes import Axes
from .axis import Tick
from .spines import Spine

class _ColorbarSpine(Spine):
    def __init__(self, axes) -> None: ...
    def get_window_extent(self, renderer: RendererBase=...): ...
    def set_xy(self, xy: Sequence[float])-> None: ...
    def draw(self, renderer)-> None: ...

class _ColorbarAxesLocator:
    def __init__(self, cbar) -> None: ...
    def __call__(self, ax, renderer): ...
    def get_subplotspec(self)-> SubplotSpec: ...

class Colorbar:
    ax: Axes
    lines: list[LineCollection]
    dividers: LineCollection
    n_rasterize: int = ...

    def __init__(
        self,
        ax: Axes,
        mappable: ScalarMappable = ...,
        *,
        cmap: Colormap = ...,
        norm: Normalize = ...,
        alpha: float = ...,
        values: None = ...,
        boundaries: None = ...,
        orientation: Literal["vertical", "horizontal"] = 'vertical',
        ticklocation: Literal["auto", "left", "right", "top", "bottom"] = 'auto',
        extend: Literal["neither", "both", "min", "max"]|None = None,
        spacing: Literal["uniform", "proportional"] = 'uniform',
        ticks: None | Sequence[Tick] | Locator = None,
        format: None | str | Formatter = None,
        drawedges: bool = False,
        filled: bool = True,
        extendfrac=None,
        extendrect: bool = False,
        label: str = ''
    ) -> None: ...
    @property
    def locator(self)-> Locator: ...
    @locator.setter
    def locator(self, loc: Locator)-> None: ...
    @property
    def minorlocator(self)-> Locator: ...
    @minorlocator.setter
    def minorlocator(self, loc: Locator)-> None: ...
    @property
    def formatter(self)-> Formatter: ...
    @formatter.setter
    def formatter(self, fmt: Formatter)-> None: ...
    @property
    def minorformatter(self)-> Formatter: ...
    @minorformatter.setter
    def minorformatter(self, fmt: Formatter)-> None: ...

    def update_normal(self, mappable)-> None: ...
    def draw_all(self)-> None: ...
    def add_lines(self, *args, **kwargs)-> None: ...
    def update_ticks(self)-> None: ...
    def set_ticks(
        self,
        ticks: list[float],
        update_ticks: bool=True,
        labels: list[str]|None = None,
        *,
        minor: bool = False,
        **kwargs
    )-> None: ...
    def get_ticks(self, minor: bool = False)-> list: ...
    def set_ticklabels(
        self, ticklabels: Text, update_ticks: bool = True, *, minor: bool=False, **kwargs
    )-> None: ...
    def minorticks_on(self)-> None: ...
    def minorticks_off(self)-> None: ...
    def set_label(self, label: str, *, loc: str|None = None, **kwargs)-> None: ...
    def set_alpha(self, alpha: float)-> None: ...
    def remove(self)-> None: ...
    def drag_pan(self, button, key, x, y)-> None: ...

ColorbarBase: Type[Colorbar] = Colorbar

def make_axes(
    parents: Axes | list[Axes],
    location: None | Literal["left", "right", "top", "bottom"] = None,
    orientation: None | Literal["vertical", "horizontal"] = None,
    fraction: float = 0.15,
    shrink: float = 1.0,
    aspect: float = 20,
    **kwargs
)-> tuple[Axes, dict]: ...
def make_axes_gridspec(
    parent: Axes,
    *,
    location: None | Literal["left", "right", "top", "bottom"] = None,
    orientation: None | Literal["vertical", "horizontal"] = None,
    fraction: float = 0.15,
    shrink: float = 1.0,
    aspect: float = 20,
    **kwargs
)-> tuple[Axes, dict]: ...



from ..metrics import DistanceMetric as _DistanceMetric

# TODO: Remove this file in 1.3
import warnings


class DistanceMetric(_DistanceMetric):
    @classmethod
    def get_metric(cls, metric, **kwargs):
        ...



import base64

import numpy as np

from . import _marching_cubes_lewiner_luts as mcluts
from ._marching_cubes_classic import _marching_cubes_classic

def marching_cubes(
    volume,
    level: float | None = None,
    *,
    spacing=...,
    gradient_direction: str = "descent",
    step_size: int = 1,
    allow_degenerate: bool = True,
    method: str = "lewiner",
    mask=None
): ...
def _marching_cubes_lewiner(
    volume,
    level,
    spacing,
    gradient_direction,
    step_size,
    allow_degenerate,
    use_classic,
    mask,
): ...
def _to_array(args): ...

# Map an edge-index to two relative pixel positions. The ege index
# represents a point that lies somewhere in between these pixels.
# Linear interpolation should be used to determine where it is exactly.
#   0
# 3   1   ->  0x
#   2         xx
EDGETORELATIVEPOSX = ...
EDGETORELATIVEPOSY = ...
EDGETORELATIVEPOSZ = ...

def _get_mc_luts(): ...



from scipy import sparse as sp
from numpy import ndarray
from ...utils.multiclass import type_of_target as type_of_target
from ...utils.validation import (
    check_array as check_array,
    check_consistent_length as check_consistent_length,
)
from math import log as log
from scipy.sparse import spmatrix
from ..._typing import ArrayLike, Float, MatrixLike
from ._expected_mutual_info_fast import (
    expected_mutual_information as expected_mutual_information,
)

# Authors: Olivier Grisel <olivier.grisel@ensta.org>
#          Wei LI <kuantkid@gmail.com>
#          Diego Molla <dmolla-aliod@gmail.com>
#          Arnaud Fouchet <foucheta@gmail.com>
#          Thierry Guillemot <thierry.guillemot.work@gmail.com>
#          Gregory Stupp <stuppie@gmail.com>
#          Joel Nothman <joel.nothman@gmail.com>
#          Arya McCarthy <arya@jhu.edu>
#          Uwe F Mayer <uwe_f_mayer@yahoo.com>
# License: BSD 3 clause


import warnings

import numpy as np


def check_clusterings(
    labels_true: ArrayLike, labels_pred: ArrayLike
) -> tuple[ndarray, ndarray]:
    ...


def contingency_matrix(
    labels_true: ArrayLike,
    labels_pred: ArrayLike,
    *,
    eps: None | Float = None,
    sparse: bool = False,
    dtype: float | int = ...
) -> ndarray | spmatrix:
    ...


# clustering measures


def pair_confusion_matrix(labels_true: ArrayLike, labels_pred: ArrayLike) -> ndarray:
    ...


def rand_score(labels_true: ArrayLike, labels_pred: ArrayLike) -> Float:
    ...


def adjusted_rand_score(labels_true: ArrayLike, labels_pred: ArrayLike) -> float:
    ...


def homogeneity_completeness_v_measure(
    labels_true: ArrayLike, labels_pred: ArrayLike, *, beta: Float = 1.0
) -> tuple[float, float, float] | tuple[Float, Float, Float]:
    ...


def homogeneity_score(labels_true: ArrayLike, labels_pred: ArrayLike) -> Float:
    ...


def completeness_score(labels_true: ArrayLike, labels_pred: ArrayLike) -> Float:
    ...


def v_measure_score(
    labels_true: ArrayLike, labels_pred: ArrayLike, *, beta: Float = 1.0
) -> Float:
    ...


def mutual_info_score(
    labels_true: None | ArrayLike,
    labels_pred: None | ArrayLike,
    *,
    contingency: None | MatrixLike = None
) -> Float:
    ...


def adjusted_mutual_info_score(
    labels_true: ArrayLike,
    labels_pred: ArrayLike,
    *,
    average_method: str = "arithmetic"
) -> Float:
    ...


def normalized_mutual_info_score(
    labels_true: ArrayLike,
    labels_pred: ArrayLike,
    *,
    average_method: str = "arithmetic"
) -> Float:
    ...


def fowlkes_mallows_score(
    labels_true: ArrayLike, labels_pred: ArrayLike, *, sparse: bool = False
) -> float:
    ...


def entropy(labels: ArrayLike) -> Float:
    ...



from typing import Any

from django.contrib.gis.db.backends.base.features import (
    BaseSpatialFeatures as BaseSpatialFeatures,
)
from django.db.backends.sqlite3.features import (
    DatabaseFeatures as SQLiteDatabaseFeatures,
)

class DatabaseFeatures(BaseSpatialFeatures, SQLiteDatabaseFeatures):
    supports_3d_storage: bool = ...
    def supports_area_geodetic(self) -> Any: ...  # type: ignore [override]



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

# Class adapted from mne-python

class SimpleBunch(dict):
    def __init__(self, **kwargs): ...



from ..exceptions import ConvergenceWarning as ConvergenceWarning
from .fixes import (
    line_search_wolfe1 as line_search_wolfe1,
    line_search_wolfe2 as line_search_wolfe2,
)

# This is a modified file from scipy.optimize
# Original authors: Travis Oliphant, Eric Jones
# Modifications by Gael Varoquaux, Mathieu Blondel and Tom Dupre la Tour
# License: BSD

import numpy as np
import warnings


class _LineSearchError(RuntimeError):
    ...



from numpy.typing import ArrayLike

#
# Anti-Grain Geometry - Version 2.4
# Copyright (C) 2002-2005 Maxim Shemanarev (McSeem)
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#   1. Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#
#   2. Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in
#      the documentation and/or other materials provided with the
#      distribution.
#
#   3. The name of the author may not be used to endorse or promote
#      products derived from this software without specific prior
#      written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
# IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# ----------------------------------------------------------------------------
#
# Python translation by Nicolas P. Rougier
# Copyright (C) 2013 Nicolas P. Rougier. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY NICOLAS P. ROUGIER ''AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL NICOLAS P. ROUGIER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# The views and conclusions contained in the software and documentation are
# those of the authors and should not be interpreted as representing official
# policies, either expressed or implied, of Nicolas P. Rougier.
#

import math

import numpy as np

curve_distance_epsilon: float = ...
curve_collinearity_epsilon: float = ...
curve_angle_tolerance_epsilon: float = ...
curve_recursion_limit: int = ...
m_cusp_limit: float = ...
m_angle_tolerance = ...
m_approximation_scale: float = ...
m_distance_tolerance_square = ...

def calc_sq_distance(x1, y1, x2, y2): ...
def _curve3_recursive_bezier(points, x1, y1, x2, y2, x3, y3, level=0): ...
def _curve4_recursive_bezier(points, x1, y1, x2, y2, x3, y3, x4, y4, level=0): ...
def curve3_bezier(p1: ArrayLike, p2: ArrayLike, p3: ArrayLike) -> ArrayLike: ...
def curve4_bezier(p1: ArrayLike, p2: ArrayLike, p3: ArrayLike, p4: ArrayLike) -> ArrayLike: ...



from typing import Any, Callable
from traceback import format_exc as format_exc
from collections import Counter as Counter
from ..base import BaseEstimator
from numpy import ndarray
from ..utils.multiclass import type_of_target as type_of_target
from functools import partial as partial
from .cluster import (
    adjusted_rand_score as adjusted_rand_score,
    rand_score as rand_score,
    homogeneity_score as homogeneity_score,
    completeness_score as completeness_score,
    v_measure_score as v_measure_score,
    mutual_info_score as mutual_info_score,
    adjusted_mutual_info_score as adjusted_mutual_info_score,
    normalized_mutual_info_score as normalized_mutual_info_score,
    fowlkes_mallows_score as fowlkes_mallows_score,
)
from ..base import is_regressor as is_regressor
from .._typing import Float, MatrixLike, ArrayLike
from collections.abc import Iterable as Iterable
from . import (
    r2_score as r2_score,
    median_absolute_error as median_absolute_error,
    max_error as max_error,
    mean_absolute_error as mean_absolute_error,
    mean_squared_error as mean_squared_error,
    mean_squared_log_error as mean_squared_log_error,
    mean_poisson_deviance as mean_poisson_deviance,
    mean_gamma_deviance as mean_gamma_deviance,
    accuracy_score as accuracy_score,
    top_k_accuracy_score as top_k_accuracy_score,
    f1_score as f1_score,
    roc_auc_score as roc_auc_score,
    average_precision_score as average_precision_score,
    precision_score as precision_score,
    recall_score as recall_score,
    log_loss as log_loss,
    balanced_accuracy_score as balanced_accuracy_score,
    explained_variance_score as explained_variance_score,
    brier_score_loss as brier_score_loss,
    jaccard_score as jaccard_score,
    mean_absolute_percentage_error as mean_absolute_percentage_error,
    matthews_corrcoef as matthews_corrcoef,
    class_likelihood_ratios as class_likelihood_ratios,
)

import numpy as np
import copy
import warnings


class _MultimetricScorer:
    def __init__(self, *, scorers: dict, raise_exc: bool = True) -> None:
        ...

    def __call__(self, estimator, *args, **kwargs) -> dict[str, Float]:
        ...


class _BaseScorer:
    def __init__(self, score_func: Callable, sign: int, kwargs) -> None:
        ...

    def __repr__(self) -> str:
        ...

    def __call__(
        self,
        estimator: Any,
        X: MatrixLike | ArrayLike,
        y_true: ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> Float:
        ...


class _PredictScorer(_BaseScorer):
    ...


class _ProbaScorer(_BaseScorer):
    ...


class _ThresholdScorer(_BaseScorer):
    ...


def get_scorer(scoring: _PredictScorer | None | str | Callable) -> Callable:
    ...


def check_scoring(
    estimator: BaseEstimator,
    scoring: _PredictScorer | None | str | Callable = None,
    *,
    allow_none: bool = False,
) -> Callable:
    ...


def make_scorer(
    score_func: Callable,
    *,
    greater_is_better: bool = True,
    needs_proba: bool = False,
    needs_threshold: bool = False,
    **kwargs,
) -> Callable:
    ...


# Standard regression scores
explained_variance_scorer = ...
r2_scorer = ...
max_error_scorer = ...
neg_mean_squared_error_scorer = ...
neg_mean_squared_log_error_scorer = ...
neg_mean_absolute_error_scorer = ...
neg_mean_absolute_percentage_error_scorer = ...
neg_median_absolute_error_scorer = ...
neg_root_mean_squared_error_scorer = ...
neg_mean_poisson_deviance_scorer = ...

neg_mean_gamma_deviance_scorer = ...

# Standard Classification Scores
accuracy_scorer = ...
balanced_accuracy_scorer = ...
matthews_corrcoef_scorer = ...


def positive_likelihood_ratio(y_true, y_pred):
    ...


def negative_likelihood_ratio(y_true, y_pred):
    ...


positive_likelihood_ratio_scorer = ...
neg_negative_likelihood_ratio_scorer = ...

# Score functions that need decision values
top_k_accuracy_scorer = ...
roc_auc_scorer = ...
average_precision_scorer = ...
roc_auc_ovo_scorer = ...
roc_auc_ovo_weighted_scorer = ...
roc_auc_ovr_scorer = ...
roc_auc_ovr_weighted_scorer = ...

# Score function for probabilistic classification
neg_log_loss_scorer = ...
neg_brier_score_scorer = ...
brier_score_loss_scorer = ...


# Clustering scores
adjusted_rand_scorer = ...
rand_scorer = ...
homogeneity_scorer = ...
completeness_scorer = ...
v_measure_scorer = ...
mutual_info_scorer = ...
adjusted_mutual_info_scorer = ...
normalized_mutual_info_scorer = ...
fowlkes_mallows_scorer = ...


# TODO(1.3) Remove
class _DeprecatedScorers(dict):
    def __getitem__(self, item):
        ...


_SCORERS = ...


def get_scorer_names() -> list[str] | ndarray:
    ...


SCORERS = ...



from numpy.typing import ArrayLike
from ..classes.digraph import DiGraph
from collections.abc import Iterable
from typing import Mapping
from ..classes.reportviews import NodeView, EdgeView, DegreeView
from ..classes.graph import Graph
from copy import deepcopy
from functools import cached_property

from ..classes.graph import Graph
import networkx.convert as convert
from ..classes.coreviews import AdjacencyView
from ..classes.reportviews import DegreeView, EdgeView, NodeView
from ..exception import NetworkXError

__all__ = ["Graph"]

class _CachedPropertyResetterAdj:
    def __set__(self, obj, value): ...

class _CachedPropertyResetterNode:
    def __set__(self, obj, value): ...

class Graph:

    _adj = ...
    _node = ...

    node_dict_factory = ...
    node_attr_dict_factory = ...
    adjlist_outer_dict_factory = ...
    adjlist_inner_dict_factory = ...
    edge_attr_dict_factory = ...
    graph_attr_dict_factory = ...

    def to_directed_class(self): ...
    def to_undirected_class(self): ...
    def __init__(self, incoming_graph_data=None, **attr): ...
    @cached_property
    def adj(self): ...
    @property
    def name(self): ...
    @name.setter
    def name(self, s): ...
    def __str__(self) -> str: ...
    def __iter__(self): ...
    def __contains__(self, n): ...
    def __len__(self) -> int: ...
    def __getitem__(self, n) -> Mapping: ...
    def add_node(self, node_for_adding, **attr): ...
    def add_nodes_from(self, nodes_for_adding, **attr): ...
    def remove_node(self, n): ...
    def remove_nodes_from(self, nodes): ...
    @cached_property
    def nodes(self) -> NodeView: ...
    def number_of_nodes(self) -> int: ...
    def order(self) -> int: ...
    def has_node(self, n): ...
    def add_edge(self, u_of_edge, v_of_edge, **attr): ...
    def add_edges_from(self, ebunch_to_add, **attr): ...
    def add_weighted_edges_from(self, ebunch_to_add, weight="weight", **attr): ...
    def remove_edge(self, u, v): ...
    def remove_edges_from(self, ebunch): ...
    def update(self, edges=None, nodes=None): ...
    def has_edge(self, u, v) -> bool: ...
    def neighbors(self, n): ...
    @cached_property
    def edges(self) -> EdgeView: ...
    def get_edge_data(self, u, v, default=None) -> Mapping: ...
    def adjacency(self): ...
    @cached_property
    def degree(self) -> DegreeView | int: ...
    def clear(self): ...
    def clear_edges(self): ...
    def is_multigraph(self): ...
    def is_directed(self): ...
    def copy(self, as_view=False) -> Graph: ...
    def to_directed(self, as_view=False) -> DiGraph: ...
    def to_undirected(self, as_view=False): ...
    def subgraph(self, nodes: ArrayLike | Iterable): ...
    def edge_subgraph(self, edges: Iterable) -> Graph: ...
    def size(self, weight=None): ...
    def number_of_edges(self, u=None, v=None) -> int: ...
    def nbunch_iter(self, nbunch=None): ...
